@@ -1,41 +1,27 @@
 //-*-Mode:java;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 // ex: set ft=java fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 //
-// BSD LICENSE
-// 
-// Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-// All rights reserved.
-// 
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-// 
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above copyright
-//       notice, this list of conditions and the following disclaimer in
-//       the documentation and/or other materials provided with the
-//       distribution.
-//     * All advertising materials mentioning features or use of this
-//       software must display the following acknowledgment:
-//         This product includes software developed by Michael Truog
-//     * The name of the author may not be used to endorse or promote
-//       products derived from this software without specific prior
-//       written permission
-// 
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-// DAMAGE.
+// MIT License
+//
+// Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the "Software"),
+// to deal in the Software without restriction, including without limitation
+// the rights to use, copy, modify, merge, publish, distribute, sublicense,
+// and/or sell copies of the Software, and to permit persons to whom the
+// Software is furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+// DEALINGS IN THE SOFTWARE.
 //
 
 package org.cloudi;
@@ -97,7 +83,7 @@ public class API
     }
     private static class NullResponse implements FunctionInterface9
     {
-        public Object invoke(Integer command, String name, String pattern,
+        public Object invoke(Integer request_type, String name, String pattern,
                              byte[] request_info, byte[] request,
                              Integer timeout, Byte priority,
                              byte[] trans_id, OtpErlangPid pid)
@@ -134,8 +120,6 @@ public class API
     private final HashMap<String, LinkedList<FunctionInterface9>> callbacks;
     private final FunctionInterface9 null_response;
     private final int buffer_size;
-    private long request_timer;
-    private Integer request_timeout;
     private int process_index;
     private int process_count;
     private int process_count_max;
@@ -146,7 +130,6 @@ public class API
     private int timeout_sync;
     private int timeout_terminate;
     private byte priority_default;
-    private boolean request_timeout_adjustment;
 
     public API(final int thread_index)
         throws InvalidInputException,
@@ -222,14 +205,14 @@ public class API
     }
 
     /**
-     * Subscribes an object method to a service name pattern. 
+     * Subscribes an object method to a service name pattern.
      *
      * @param  pattern     the service name pattern
      * @param  instance    the object instance
      * @param  methodName  the object method to handle matching requests
      *
      * @throws NoSuchMethodException instance method arity is invalid
-     */ 
+     */
     public void subscribe(final String pattern,
                           final Object instance,
                           final String methodName)
@@ -239,11 +222,11 @@ public class API
     }
 
     /**
-     * Subscribes an object method to a service name pattern. 
+     * Subscribes an object method to a service name pattern.
      *
      * @param  pattern     the service name pattern
      * @param  callback    method reference for callback (Java 8 or higher)
-     */ 
+     */
     public void subscribe(final String pattern,
                           final FunctionInterface9 callback)
     {
@@ -275,7 +258,7 @@ public class API
      * @return count of active subscriptions
      * @throws InvalidInputException invalid input to internal function call
      * @throws TerminateException service execution must terminate
-     */ 
+     */
     public int subscribe_count(final String pattern)
         throws InvalidInputException,
                TerminateException
@@ -298,7 +281,7 @@ public class API
     }
 
     /**
-     * Unsubscribes from a service name pattern. 
+     * Unsubscribes from a service name pattern.
      *
      * @param pattern  the service name pattern
      *
@@ -535,7 +518,7 @@ public class API
      * Forward a message to another service
      * subscribed that matches the destination service <code>name</code>.
      *
-     * @param command       constant API.SYNC or API.ASYNC
+     * @param request_type  constant API.SYNC or API.ASYNC
      * @param name          the destination service name
      * @param request_info  any request metadata
      * @param request       the request data
@@ -548,7 +531,7 @@ public class API
      * @throws ForwardSyncException sync service request was forwarded
      * @throws InvalidInputException invalid service request type
      */
-    public void forward_(final Integer command,
+    public void forward_(final Integer request_type,
                          final String name,
                          final byte[] request_info,
                          final byte[] request,
@@ -560,10 +543,10 @@ public class API
                ForwardSyncException,
                InvalidInputException
     {
-        if (command == API.ASYNC)
+        if (request_type == API.ASYNC)
             forward_async(name, request_info, request,
                           timeout, priority, trans_id, pid);
-        else if (command == API.SYNC)
+        else if (request_type == API.SYNC)
             forward_sync(name, request_info, request,
                          timeout, priority, trans_id, pid);
         else
@@ -593,22 +576,6 @@ public class API
                               final OtpErlangPid pid)
         throws ForwardAsyncException
     {
-        if (this.request_timeout_adjustment)
-        {
-            if (timeout == this.request_timeout)
-            {
-                final int elapsed = (int) Math.max(0,
-                    ((System.nanoTime() - this.request_timer) * 1e-6));
-                if (elapsed > timeout)
-                {
-                    timeout = 0;
-                }
-                else
-                {
-                    timeout -= elapsed;
-                }
-            }
-        }
         try
         {
             OtpOutputStream forward_async = new OtpOutputStream();
@@ -655,22 +622,6 @@ public class API
                              final OtpErlangPid pid)
         throws ForwardSyncException
     {
-        if (this.request_timeout_adjustment)
-        {
-            if (timeout == this.request_timeout)
-            {
-                final int elapsed = (int) Math.max(0,
-                    ((System.nanoTime() - this.request_timer) * 1e-6));
-                if (elapsed > timeout)
-                {
-                    timeout = 0;
-                }
-                else
-                {
-                    timeout -= elapsed;
-                }
-            }
-        }
         try
         {
             OtpOutputStream forward_sync = new OtpOutputStream();
@@ -697,7 +648,7 @@ public class API
     /**
      * Returns a response from a service request.
      *
-     * @param command        constant API.SYNC or API.SYNC
+     * @param request_type   constant API.SYNC or API.SYNC
      * @param name           the service name
      * @param pattern        the service name pattern
      * @param response_info  any response metadata
@@ -710,7 +661,7 @@ public class API
      * @throws ReturnSyncException sync service request returned
      * @throws InvalidInputException invalid service request type
      */
-    public void return_(final Integer command,
+    public void return_(final Integer request_type,
                         final String name,
                         final String pattern,
                         final byte[] response_info,
@@ -722,10 +673,10 @@ public class API
                ReturnSyncException,
                InvalidInputException
     {
-        if (command == API.ASYNC)
+        if (request_type == API.ASYNC)
             return_async(name, pattern, response_info, response,
                          timeout, trans_id, pid);
-        else if (command == API.SYNC)
+        else if (request_type == API.SYNC)
             return_sync(name, pattern, response_info, response,
                         timeout, trans_id, pid);
         else
@@ -754,24 +705,6 @@ public class API
                              final OtpErlangPid pid)
         throws ReturnAsyncException
     {
-        if (this.request_timeout_adjustment)
-        {
-            if (timeout == this.request_timeout)
-            {
-                final int elapsed = (int) Math.max(0,
-                    ((System.nanoTime() - this.request_timer) * 1e-6));
-                if (elapsed > timeout)
-                {
-                    response_info = new byte[0];
-                    response = new byte[0];
-                    timeout = 0;
-                }
-                else
-                {
-                    timeout -= elapsed;
-                }
-            }
-        }
         try
         {
             OtpOutputStream return_async = new OtpOutputStream();
@@ -817,24 +750,6 @@ public class API
                             final OtpErlangPid pid)
         throws ReturnSyncException
     {
-        if (this.request_timeout_adjustment)
-        {
-            if (timeout == this.request_timeout)
-            {
-                final int elapsed = (int) Math.max(0,
-                    ((System.nanoTime() - this.request_timer) * 1e-6));
-                if (elapsed > timeout)
-                {
-                    response_info = new byte[0];
-                    response = new byte[0];
-                    timeout = 0;
-                }
-                else
-                {
-                    timeout -= elapsed;
-                }
-            }
-        }
         try
         {
             OtpOutputStream return_sync = new OtpOutputStream();
@@ -1094,12 +1009,6 @@ public class API
                MessageDecodingException,
                TerminateException
     {
-        long request_time_start = 0;
-        if (this.request_timeout_adjustment)
-        {
-            this.request_timer = System.nanoTime();
-            this.request_timeout = timeout;
-        }
         LinkedList<FunctionInterface9> callback_list =
             this.callbacks.get(pattern);
         FunctionInterface9 callback = null;
@@ -1120,7 +1029,14 @@ public class API
                                                   request_info, request,
                                                   timeout, priority,
                                                   trans_id, pid);
-                if (response.getClass() == byte[][].class)
+                if (response == null)
+                {
+                    return_async(name, pattern,
+                                 ("").getBytes(),
+                                 ("").getBytes(),
+                                 timeout, trans_id, pid);
+                }
+                else if (response.getClass() == byte[][].class)
                 {
                     byte [][] response_array = (byte[][]) response;
                     assert response_array.length == 2 : "invalid response";
@@ -1128,7 +1044,7 @@ public class API
                                  response_array[0],
                                  response_array[1],
                                  timeout, trans_id, pid);
-                    
+
                 }
                 else if (response.getClass() == byte[].class)
                 {
@@ -1190,7 +1106,14 @@ public class API
                                                   request_info, request,
                                                   timeout, priority,
                                                   trans_id, pid);
-                if (response.getClass() == byte[][].class)
+                if (response == null)
+                {
+                    return_sync(name, pattern,
+                                ("").getBytes(),
+                                ("").getBytes(),
+                                timeout, trans_id, pid);
+                }
+                else if (response.getClass() == byte[][].class)
                 {
                     byte [][] response_array = (byte[][]) response;
                     assert response_array.length == 2 : "invalid response";
@@ -1198,7 +1121,7 @@ public class API
                                 response_array[0],
                                 response_array[1],
                                 timeout, trans_id, pid);
-                    
+
                 }
                 else if (response.getClass() == byte[].class)
                 {
@@ -1292,6 +1215,9 @@ public class API
                 case MESSAGE_REINIT:
                 {
                     this.process_count = buffer.getInt();
+                    this.timeout_async = buffer.getInt();
+                    this.timeout_sync = buffer.getInt();
+                    this.priority_default = buffer.get();
                     break;
                 }
                 case MESSAGE_KEEPALIVE:
@@ -1366,8 +1292,6 @@ public class API
                         this.timeout_sync = buffer.getInt();
                         this.timeout_terminate = buffer.getInt();
                         this.priority_default = buffer.get();
-                        this.request_timeout_adjustment =
-                            (buffer.get() & 0xFF) != 0;
                         if (buffer.hasRemaining())
                         {
                             assert ! external;
@@ -1468,6 +1392,9 @@ public class API
                     case MESSAGE_REINIT:
                     {
                         this.process_count = buffer.getInt();
+                        this.timeout_async = buffer.getInt();
+                        this.timeout_sync = buffer.getInt();
+                        this.priority_default = buffer.get();
                         if (buffer.hasRemaining())
                             continue;
                         break;
@@ -1573,12 +1500,6 @@ public class API
         return result;
     }
 
-    public HashMap<String,
-                   ArrayList<String>> request_http_qs_parse(final byte[] r)
-    {
-        return binary_key_value_parse(r);
-    }
-
     public HashMap<String,
                    ArrayList<String>> info_key_value_parse(final byte[] info)
     {
@@ -1611,20 +1532,9 @@ public class API
                             final Integer timeout)
         throws IOException
     {
-        final ByteArrayOutputStream output =
-            new ByteArrayOutputStream(this.buffer_size);
-        int i = 0;
-        if (buffer_in != null && buffer_in.hasRemaining())
-        {
-            i += buffer_in.limit() - buffer_in.position();
-            output.write(buffer_in.array(),
-                         buffer_in.position(),
-                         buffer_in.limit());
-        }
-        int read = 0;
-        final byte[] bytes = new byte[this.buffer_size];
-        boolean consume;
-        if (i == 0 && timeout != null)
+        final boolean buffer_in_data =
+            (buffer_in != null && buffer_in.hasRemaining());
+        if (! buffer_in_data && timeout != null)
         {
             // simulate poll
             final FileInputStream input = this.input;
@@ -1654,12 +1564,70 @@ public class API
                 throw new IOException("poll exception");
             }
         }
+        int read = 0;
+        final byte[] bytes = new byte[this.buffer_size];
+        ByteBuffer buffer_out = null;
         if (this.use_header)
-            consume = (i < 4);
-        else
-            consume = (i < this.buffer_size);
-        while (consume)
         {
+            final byte[] header = new byte[4];
+            int header_i = 0;
+            while (header_i < 4)
+            {
+                read = this.input.read(header, header_i, 4 - header_i);
+                if (read == -1)
+                {
+                    throw new IOException("consume read eof");
+                }
+                else if (read > 0)
+                {
+                    header_i += read;
+                }
+            }
+            final int length = ((header[0] & 0xff) << 24) |
+                               ((header[1] & 0xff) << 16) |
+                               ((header[2] & 0xff) <<  8) |
+                               (header[3] & 0xff);
+            if (length < 0)
+                throw new IOException("negative length");
+            int total = length;
+            if (buffer_in_data)
+            {
+                total += buffer_in.limit() - buffer_in.position();
+            }
+            buffer_out = ByteBuffer.allocate(total);
+            if (buffer_in_data)
+            {
+                buffer_out.put(buffer_in);
+            }
+            int i = 0;
+            while (i < length)
+            {
+                read = this.input.read(bytes, 0,
+                                       Math.min(length - i, this.buffer_size));
+                if (read == -1)
+                {
+                    throw new IOException("remaining read eof");
+                }
+                else if (read > 0)
+                {
+                    i += read;
+                    buffer_out.put(bytes, 0, read);
+                }
+            }
+            buffer_out.rewind();
+        }
+        else
+        {
+            final ByteArrayOutputStream output =
+                new ByteArrayOutputStream(this.buffer_size);
+            int i = 0;
+            if (buffer_in_data)
+            {
+                i += buffer_in.limit() - buffer_in.position();
+                output.write(buffer_in.array(),
+                             buffer_in.position(),
+                             i);
+            }
             while ((read = this.input.read(bytes)) == this.buffer_size &&
                    this.input.available() > 0)
             {
@@ -1675,50 +1643,7 @@ public class API
                 i += read;
                 output.write(bytes, 0, read);
             }
-            if (this.use_header == false)
-                consume = false;
-            else if (i >= 4)
-                consume = false;
-        }
-        final byte[] result = output.toByteArray();
-        ByteBuffer buffer_out = null;
-        if (this.use_header)
-        {
-            final int length = ((result[0] & 0xff) << 24) |
-                               ((result[1] & 0xff) << 16) |
-                               ((result[2] & 0xff) <<  8) |
-                                (result[3] & 0xff);
-            if (length < 0)
-                throw new IOException("negative length");
-            i -= 4;
-            if (i < length)
-            {
-                buffer_out = ByteBuffer.allocate(length);
-                buffer_out.put(result, 4, i);
-                while (i < length)
-                {
-                    read = this.input.read(bytes, 0,
-                                      Math.min(length - i, this.buffer_size));
-                    if (read == -1)
-                    {
-                        throw new IOException("remaining read eof");
-                    }
-                    else if (read > 0)
-                    {
-                        i += read;
-                        buffer_out.put(bytes, 0, read);
-                    }
-                }
-                buffer_out.rewind();
-            }
-            else
-            {
-                buffer_out = ByteBuffer.wrap(result, 4, i);
-            }
-        }
-        else
-        {
-            buffer_out = ByteBuffer.wrap(result);
+            buffer_out = ByteBuffer.wrap(output.toByteArray());
         }
         buffer_out.order(ByteOrder.nativeOrder());
         return buffer_out;
@@ -1775,7 +1700,7 @@ public class API
         FileDescriptor object;
         try
         {
-            object = c.newInstance(new Integer(fd));
+            object = c.newInstance(Integer.valueOf(fd));
         }
         catch (IllegalArgumentException e)
         {
@@ -1952,7 +1877,7 @@ public class API
             super("Synchronous Call Return Invalid");
         }
     }
-    
+
     public static class ReturnAsyncException extends Exception
     {
         private static final long serialVersionUID = 3L;
@@ -1970,7 +1895,7 @@ public class API
             super("Synchronous Call Forward Invalid");
         }
     }
-    
+
     public static class ForwardAsyncException extends Exception
     {
         private static final long serialVersionUID = 3L;
