@@ -2,41 +2,27 @@
 #-*-Mode:perl;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 # ex: set ft=perl fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
-# BSD LICENSE
-# 
-# Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-# 
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in
-#       the documentation and/or other materials provided with the
-#       distribution.
-#     * All advertising materials mentioning features or use of this
-#       software must display the following acknowledgment:
-#         This product includes software developed by Michael Truog
-#     * The name of the author may not be used to endorse or promote
-#       products derived from this software without specific prior
-#       written permission
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-# DAMAGE.
+# MIT License
+#
+# Copyright (c) 2014-2018 Michael Truog <mjtruog at gmail dot com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the "Software"),
+# to deal in the Software without restriction, including without limitation
+# the rights to use, copy, modify, merge, publish, distribute, sublicense,
+# and/or sell copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
 #
 
 package Erlang;
@@ -44,7 +30,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$Erlang::VERSION = '1.53';
+$Erlang::VERSION = '1.73';
 
 require Compress::Zlib;
 # bigint/bignum caused slowness without enough features to be useful
@@ -92,6 +78,8 @@ require Erlang::ParseException;
 require Erlang::InputException;
 require Erlang::OutputException;
 
+# core functionality
+
 sub binary_to_term
 {
     my ($data) = @_;
@@ -158,11 +146,17 @@ sub term_to_binary
             die Erlang::InputException->new('compression failed');
         }
         my $size_uncompressed = length($data_uncompressed);
+        if ($size_uncompressed > 4294967295)
+        {
+            die Erlang::OutputException->new('uint32 overflow');
+        }
         return pack('CCN', TAG_VERSION, TAG_COMPRESSED_ZLIB,
                     $size_uncompressed) . $data_compressed;
     }
 }
 
+# binary_to_term implementation functions
+
 sub _binary_to_term
 {
     no warnings 'all';
@@ -249,18 +243,18 @@ sub _binary_to_term
     }
     elsif ($tag == TAG_SMALL_TUPLE_EXT || $tag == TAG_LARGE_TUPLE_EXT)
     {
-        my $arity;
+        my $length;
         if ($tag == TAG_SMALL_TUPLE_EXT)
         {
-            $arity = ord(substr($data, $i, 1));
+            $length = ord(substr($data, $i, 1));
             $i += 1;
         }
         elsif ($tag == TAG_LARGE_TUPLE_EXT)
         {
-            ($arity) = unpack('N', substr($data, $i, 4));
+            ($length) = unpack('N', substr($data, $i, 4));
             $i += 4;
         }
-        return _binary_to_term_sequence($i, $arity, $data);
+        return _binary_to_term_sequence($i, $length, $data);
     }
     elsif ($tag == TAG_NIL_EXT)
     {
@@ -274,10 +268,10 @@ sub _binary_to_term
     }
     elsif ($tag == TAG_LIST_EXT)
     {
-        my ($arity) = unpack('N', substr($data, $i, 4));
+        my ($length) = unpack('N', substr($data, $i, 4));
         $i += 4;
         my $tmp;
-        ($i, $tmp) = _binary_to_term_sequence($i, $arity, $data);
+        ($i, $tmp) = _binary_to_term_sequence($i, $length, $data);
         my $tail;
         ($i, $tail) = _binary_to_term($i, $data);
         if (ref($tail) ne 'Erlang::OtpErlangList' || $tail->count() != 0)
@@ -330,9 +324,10 @@ sub _binary_to_term
     }
     elsif ($tag == TAG_NEW_FUN_EXT)
     {
-        my ($size) = unpack('N', substr($data, $i, 4));
-        return ($i + $size,
-                Erlang::OtpErlangFunction->new($tag, substr($data, $i, $size)));
+        my ($length) = unpack('N', substr($data, $i, 4));
+        return ($i + $length,
+                Erlang::OtpErlangFunction->new($tag, substr($data, $i,
+                                                            $length)));
     }
     elsif ($tag == TAG_EXPORT_EXT)
     {
@@ -374,14 +369,14 @@ sub _binary_to_term
     }
     elsif ($tag == TAG_MAP_EXT)
     {
-        my ($arity) = unpack('N', substr($data, $i, 4));
+        my ($length) = unpack('N', substr($data, $i, 4));
         $i += 4;
         my %pairs = ();
-        if ($arity > 0)
+        if ($length > 0)
         {
             my $key;
             my $value;
-            for my $arity_index (0 .. ($arity - 1))
+            for my $length_index (0 .. ($length - 1))
             {
                 ($i, $key) = _binary_to_term($i, $data);
                 ($i, $value) = _binary_to_term($i, $data);
@@ -455,12 +450,12 @@ sub _binary_to_term
 sub _binary_to_term_sequence
 {
     no warnings 'all';
-    my ($i, $arity, $data) = @_;
+    my ($i, $length, $data) = @_;
     my @sequence = ();
-    if ($arity > 0)
+    if ($length > 0)
     {
         my $element;
-        for my $arity_index (0 .. ($arity - 1))
+        for my $length_index (0 .. ($length - 1))
         {
             ($i, $element) = _binary_to_term($i, $data);
             push(@sequence, $element);
@@ -469,6 +464,8 @@ sub _binary_to_term_sequence
     return ($i, \@sequence);
 }
 
+# (binary_to_term Erlang term primitive type functions)
+
 sub _binary_to_integer
 {
     no warnings 'all';
@@ -558,13 +555,19 @@ sub _binary_to_atom
     }
 }
 
+# term_to_binary implementation functions
+
 sub _term_to_binary
 {
     my ($term) = @_;
     my $ref = ref($term);
     if ($ref eq '')
     {
-        if (scalar($term) ne $term) # list
+        if (not defined($term))
+        {
+            return Erlang::OtpErlangAtom->new('undefined')->binary();
+        }
+        elsif (scalar($term) ne $term) # list
         {
             return _tuple_to_binary($term);
         }
@@ -606,50 +609,82 @@ sub _term_to_binary
     }
 }
 
+# (term_to_binary Erlang term composite type functions)
+
 sub _string_to_binary
 {
     my ($term) = @_;
-    my $arity = length($term);
-    if ($arity == 0)
+    my $length = length($term);
+    if ($length == 0)
     {
         return chr(TAG_NIL_EXT);
     }
-    elsif ($arity < 65536)
+    elsif ($length <= 65535)
     {
-        return pack('Cn', TAG_STRING_EXT, $arity) . $term;
+        return pack('Cn', TAG_STRING_EXT, $length) . $term;
     }
-    else
+    elsif ($length <= 4294967295)
     {
         my $term_packed = '';
         for my $c (split(//, $term))
         {
             $term_packed .= chr(TAG_SMALL_INTEGER_EXT) . $c;
         }
-        return pack('CN', TAG_LIST_EXT, $arity) . $term_packed .
+        return pack('CN', TAG_LIST_EXT, $length) . $term_packed .
                chr(TAG_NIL_EXT);
     }
+    else
+    {
+        die Erlang::OutputException->new('uint32 overflow');
+    }
 }
 
 sub _tuple_to_binary
 {
     my ($term_ref) = @_;
     my @term = @$term_ref;
-    my $arity = scalar(@term);
+    my $length = scalar(@term);
     my $term_packed = '';
     for my $element (@term)
     {
         $term_packed .= _term_to_binary($element);
     }
-    if ($arity < 256)
+    if ($length <= 255)
     {
-        return pack('CC', TAG_SMALL_TUPLE_EXT, $arity) . $term_packed;
+        return pack('CC', TAG_SMALL_TUPLE_EXT, $length) . $term_packed;
+    }
+    elsif ($length <= 4294967295)
+    {
+        return pack('CN', TAG_LARGE_TUPLE_EXT, $length) . $term_packed;
     }
     else
     {
-        return pack('CN', TAG_LARGE_TUPLE_EXT, $arity) . $term_packed;
+        die Erlang::OutputException->new('uint32 overflow');
     }
 }
 
+sub _hash_to_binary
+{
+    my (%term) = @_;
+    my $term_packed = '';
+    my $length = 0;
+    while (my ($key, $value) = each(%term))
+    {
+        $term_packed .= _term_to_binary($key) . _term_to_binary($value);
+        $length++;
+    }
+    if ($length <= 4294967295)
+    {
+        return pack('CN', TAG_MAP_EXT, $length) . $term_packed;
+    }
+    else
+    {
+        die Erlang::OutputException->new('uint32 overflow');
+    }
+}
+
+# (term_to_binary Erlang term primitive type functions)
+
 sub _integer_to_binary
 {
     my ($term) = @_;
@@ -671,47 +706,36 @@ sub _bignum_to_binary
 {
     my ($term) = @_;
     my $bignum = abs($term);
-    my $size = ceil(_bignum_bit_length($bignum) / 8.0);
     my $sign;
     if ($term < 0)
     {
-        $sign = chr(1);
+        $sign = 1;
     }
     else
     {
-        $sign = chr(0);
+        $sign = 0;
     }
-    my $l = $sign;
-    if ($size > 0)
+    my $l = '';
+    while ($bignum > 0)
     {
-        for my $byte (0 .. ($size - 1))
-        {
-            $l .= chr($bignum & 255);
-            $bignum = $bignum >> 8;
-        }
+        $l .= chr($bignum & 255);
+        $bignum = $bignum >> 8;
     }
-    if ($size < 256)
+    my $length = length($l);
+    if ($length <= 255)
     {
-        return pack('CC', TAG_SMALL_BIG_EXT, $size) . $l;
+        return pack('CCC', TAG_SMALL_BIG_EXT, $length, $sign) . $l;
+    }
+    elsif ($length <= 4294967295)
+    {
+        return pack('CNC', TAG_LARGE_BIG_EXT, $length, $sign) . $l;
     }
     else
     {
-        return pack('CN', TAG_LARGE_BIG_EXT, $size) . $l;
+        die Erlang::OutputException->new('uint32 overflow');
     }
 }
 
-sub _bignum_bit_length
-{
-    my ($bignum) = @_;
-    my $count = 0;
-    while ($bignum)
-    {
-        $bignum = $bignum >> 1;
-        $count++;
-    }
-    return $count;
-}
-
 sub _float_to_binary
 {
     my ($term) = @_;
@@ -725,17 +749,4 @@ sub _float_to_binary
     }
 }
 
-sub _hash_to_binary
-{
-    my (%term) = @_;
-    my $term_packed = '';
-    my $arity = 0;
-    while (my ($key, $value) = each(%term))
-    {
-        $term_packed .= _term_to_binary($key) . _term_to_binary($value);
-        $arity++;
-    }
-    return pack('CN', TAG_MAP_EXT, $arity) . $term_packed;
-}
-
 1;
