@@ -1,41 +1,27 @@
 <?php //-*-coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 // ex: set ft=php fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 //
-// BSD LICENSE
-// 
-// Copyright (c) 2014, Michael Truog <mjtruog at gmail dot com>
-// All rights reserved.
-// 
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-// 
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above copyright
-//       notice, this list of conditions and the following disclaimer in
-//       the documentation and/or other materials provided with the
-//       distribution.
-//     * All advertising materials mentioning features or use of this
-//       software must display the following acknowledgment:
-//         This product includes software developed by Michael Truog
-//     * The name of the author may not be used to endorse or promote
-//       products derived from this software without specific prior
-//       written permission
-// 
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-// DAMAGE.
+// MIT License
+//
+// Copyright (c) 2014-2018 Michael Truog <mjtruog at gmail dot com>
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the "Software"),
+// to deal in the Software without restriction, including without limitation
+// the rights to use, copy, modify, merge, publish, distribute, sublicense,
+// and/or sell copies of the Software, and to permit persons to whom the
+// Software is furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+// DEALINGS IN THE SOFTWARE.
 //
 
 namespace Erlang;
@@ -70,6 +56,8 @@ define(__NAMESPACE__ . '\TAG_FUN_EXT', 117);
 define(__NAMESPACE__ . '\TAG_ATOM_UTF8_EXT', 118);
 define(__NAMESPACE__ . '\TAG_SMALL_ATOM_UTF8_EXT', 119);
 
+// Erlang term classes listed alphabetically
+
 class OtpErlangAtom
 {
     public $value;
@@ -87,31 +75,39 @@ class OtpErlangAtom
         }
         elseif (is_string($this->value))
         {
-            $size = strlen($this->value);
+            $length = strlen($this->value);
             if ($this->utf8)
             {
-                if ($size < 256)
+                if ($length <= 255)
                 {
-                    return pack('CC', TAG_SMALL_ATOM_UTF8_EXT, $size) .
+                    return pack('CC', TAG_SMALL_ATOM_UTF8_EXT, $length) .
+                           $this->value;
+                }
+                elseif ($length <= 65535)
+                {
+                    return pack('Cn', TAG_ATOM_UTF8_EXT, $length) .
                            $this->value;
                 }
                 else
                 {
-                    return pack('Cn', TAG_ATOM_UTF8_EXT, $size) .
-                           $this->value;
+                    throw new OutputException('uint16 overflow');
                 }
             }
             else
             {
-                if ($size < 256)
+                if ($length <= 255)
                 {
-                    return pack('CC', TAG_SMALL_ATOM_EXT, $size) .
+                    return pack('CC', TAG_SMALL_ATOM_EXT, $length) .
+                           $this->value;
+                }
+                elseif ($length <= 65535)
+                {
+                    return pack('Cn', TAG_ATOM_EXT, $length) .
                            $this->value;
                 }
                 else
                 {
-                    return pack('Cn', TAG_ATOM_EXT, $size) .
-                           $this->value;
+                    throw new OutputException('uint16 overflow');
                 }
             }
         }
@@ -127,55 +123,6 @@ class OtpErlangAtom
     }
 }
 
-class OtpErlangList
-{
-    public $value;
-    public $improper;
-    public function __construct($value, $improper = false)
-    {
-        $this->value = $value;
-        $this->improper = $improper;
-    }
-    public function binary()
-    {
-        if (is_array($this->value))
-        {
-            $length = count($this->value);
-            if ($length == 0)
-            {
-                return chr(TAG_NIL_EXT);
-            }
-            elseif ($this->improper)
-            {
-                $contents = '';
-                while (list($tmp, $element) = each($this->value))
-                    $contents .= _term_to_binary($element);
-                reset($this->value);
-                return pack('CN', TAG_LIST_EXT, $length - 1) . $contents;
-            }
-            else
-            {
-                $contents = '';
-                while (list($tmp, $element) = each($this->value))
-                    $contents .= _term_to_binary($element);
-                reset($this->value);
-                return pack('CN', TAG_LIST_EXT, $length) . $contents .
-                       chr(TAG_NIL_EXT);
-            }
-        }
-        else
-        {
-            throw new OutputException('unknown list type');
-        }
-    }
-    public function __toString()
-    {
-        return sprintf('%s(array(%s),improper=%s)', get_class(),
-                       implode(',', $this->value),
-                       $this->improper ? 'true' : 'false');
-    }
-}
-
 class OtpErlangBinary
 {
     public $value;
@@ -189,15 +136,19 @@ class OtpErlangBinary
     {
         if (is_string($this->value))
         {
-            $size = strlen($this->value);
-            if ($this->bits != 8)
+            $length = strlen($this->value);
+            if ($length > 4294967295)
             {
-                return pack('CNC', TAG_BIT_BINARY_EXT, $size,
+                throw new OutputException('uint32 overflow');
+            }
+            elseif ($this->bits != 8)
+            {
+                return pack('CNC', TAG_BIT_BINARY_EXT, $length,
                             $this->bits) . $this->value;
             }
             else
             {
-                return pack('CN', TAG_BINARY_EXT, $size) . $this->value;
+                return pack('CN', TAG_BINARY_EXT, $length) . $this->value;
             }
         }
         else
@@ -232,58 +183,90 @@ class OtpErlangFunction
     }
 }
 
-class OtpErlangReference
+class OtpErlangList
 {
-    public $node;
-    public $id;
-    public $creation;
-    public function __construct($node, $id, $creation)
+    public $value;
+    public $improper;
+    public function __construct($value, $improper = false)
     {
-        $this->node = $node;
-        $this->id = $id;
-        $this->creation = $creation;
+        $this->value = $value;
+        $this->improper = $improper;
     }
     public function binary()
     {
-        $size = intval(strlen($this->id) / 4);
-        if ($size > 1)
+        if (is_array($this->value))
         {
-            return pack('Cn', TAG_NEW_REFERENCE_EXT, $size) .
-                   $this->node->binary() . $this->creation . $this->id;
+            $length = count($this->value);
+            if ($length == 0)
+            {
+                return chr(TAG_NIL_EXT);
+            }
+            elseif ($length > 4294967295)
+            {
+                throw new OutputException('uint32 overflow');
+            }
+            elseif ($this->improper)
+            {
+                $contents = '';
+                while (list($tmp, $element) = each($this->value))
+                    $contents .= _term_to_binary($element);
+                reset($this->value);
+                return pack('CN', TAG_LIST_EXT, $length - 1) . $contents;
+            }
+            else
+            {
+                $contents = '';
+                while (list($tmp, $element) = each($this->value))
+                    $contents .= _term_to_binary($element);
+                reset($this->value);
+                return pack('CN', TAG_LIST_EXT, $length) . $contents .
+                       chr(TAG_NIL_EXT);
+            }
         }
         else
         {
-            return chr(TAG_REFERENCE_EXT) .
-                   $this->node->binary() . $this->id . $this->creation;
+            throw new OutputException('unknown list type');
         }
     }
     public function __toString()
     {
-        return sprintf('%s(%s,%s,%s)', get_class(),
-                       $this->node, $this->id, $this->creation);
+        return sprintf('%s(array(%s),improper=%s)', get_class(),
+                       implode(',', $this->value),
+                       $this->improper ? 'true' : 'false');
     }
 }
 
-class OtpErlangPort
+class OtpErlangMap
 {
-    public $node;
-    public $id;
-    public $creation;
-    public function __construct($node, $id, $creation)
+    public $pairs;
+    public function __construct($pairs)
     {
-        $this->node = $node;
-        $this->id = $id;
-        $this->creation = $creation;
+        $this->pairs = $pairs;
     }
     public function binary()
     {
-        return chr(TAG_PORT_EXT) .
-               $this->node->binary() . $this->id . $this->creation;
+        $length = count($this->pairs);
+        if ($length <= 4294967295)
+        {
+            $term_packed = '';
+            foreach ($this->pairs as $pair)
+            {
+                list($key, $value) = $pair;
+                $key_packed = _term_to_binary($key);
+                $value_packed = _term_to_binary($value);
+                $term_packed .= $key_packed . $value_packed;
+            }
+            return pack('CN', TAG_MAP_EXT, $length) . $term_packed;
+        }
+        else
+        {
+            throw new OutputException('uint32 overflow');
+        }
     }
     public function __toString()
     {
-        return sprintf('%s(%s,%s,%s)', get_class(),
-                       $this->node, $this->id, $this->creation);
+        return sprintf('%s(%d)', get_class(),
+                       count($this->pairs));
     }
 }
 
@@ -312,30 +295,62 @@ class OtpErlangPid
     }
 }
 
-class OtpErlangMap
+class OtpErlangPort
 {
-    public $pairs;
-    public function __construct($pairs)
+    public $node;
+    public $id;
+    public $creation;
+    public function __construct($node, $id, $creation)
     {
-        $this->pairs = $pairs;
+        $this->node = $node;
+        $this->id = $id;
+        $this->creation = $creation;
     }
     public function binary()
     {
-        $arity = count($this->pairs);
-        $term_packed = '';
-        foreach ($this->pairs as $pair)
-        {
-            list($key, $value) = $pair;
-            $key_packed = _term_to_binary($key);
-            $value_packed = _term_to_binary($value);
-            $term_packed .= $key_packed . $value_packed;
-        }
-        return pack('CN', TAG_MAP_EXT, $arity) . $term_packed;
+        return chr(TAG_PORT_EXT) .
+               $this->node->binary() . $this->id . $this->creation;
     }
     public function __toString()
     {
-        return sprintf('%s(%d)', get_class(),
-                       count($this->pairs));
+        return sprintf('%s(%s,%s,%s)', get_class(),
+                       $this->node, $this->id, $this->creation);
+    }
+}
+
+class OtpErlangReference
+{
+    public $node;
+    public $id;
+    public $creation;
+    public function __construct($node, $id, $creation)
+    {
+        $this->node = $node;
+        $this->id = $id;
+        $this->creation = $creation;
+    }
+    public function binary()
+    {
+        $length = intval(strlen($this->id) / 4);
+        if ($length == 0)
+        {
+            return chr(TAG_REFERENCE_EXT) .
+                   $this->node->binary() . $this->id . $this->creation;
+        }
+        elseif ($length <= 65535)
+        {
+            return pack('Cn', TAG_NEW_REFERENCE_EXT, $length) .
+                   $this->node->binary() . $this->creation . $this->id;
+        }
+        else
+        {
+            throw new OutputException('uint16 overflow');
+        }
+    }
+    public function __toString()
+    {
+        return sprintf('%s(%s,%s,%s)', get_class(),
+                       $this->node, $this->id, $this->creation);
     }
 }
 
@@ -348,6 +363,8 @@ function _error_handler($errno = 0, $errstr = null,
     throw new \ErrorException($errstr, 0, $errno, $errfile, $errline);
 }
 
+// core functionality
+
 function binary_to_term($data)
 {
     if (! is_string($data))
@@ -388,11 +405,17 @@ function term_to_binary($term, $compressed = false)
             throw new InputException('compressed in [0..9]');
         $data_compressed = gzcompress($data_uncompressed, $compressed);
         $size_uncompressed = strlen($data_uncompressed);
+        if ($size_uncompressed > 4294967295)
+        {
+            throw new OutputException('uint32 overflow');
+        }
         return pack('CCN', TAG_VERSION, TAG_COMPRESSED_ZLIB,
                     $size_uncompressed) . $data_compressed;
     }
 }
 
+// binary_to_term implementation functions
+
 function _binary_to_term($i, $data)
 {
     $tag = ord($data[$i]);
@@ -451,15 +474,15 @@ function _binary_to_term($i, $data)
         case TAG_LARGE_TUPLE_EXT:
             if ($tag == TAG_SMALL_TUPLE_EXT)
             {
-                $arity = ord($data[$i]);
+                $length = ord($data[$i]);
                 $i += 1;
             }
             elseif ($tag == TAG_LARGE_TUPLE_EXT)
             {
-                list(, $arity) = unpack('N', substr($data, $i, 4));
+                list(, $length) = unpack('N', substr($data, $i, 4));
                 $i += 4;
             }
-            return _binary_to_term_sequence($i, $arity, $data);
+            return _binary_to_term_sequence($i, $length, $data);
         case TAG_NIL_EXT:
             return array($i, new OtpErlangList(array()));
         case TAG_STRING_EXT:
@@ -467,9 +490,9 @@ function _binary_to_term($i, $data)
             $i += 2;
             return array($i + $j, substr($data, $i, $j));
         case TAG_LIST_EXT:
-            list(, $arity) = unpack('N', substr($data, $i, 4));
+            list(, $length) = unpack('N', substr($data, $i, 4));
             $i += 4;
-            list($i, $tmp) = _binary_to_term_sequence($i, $arity, $data);
+            list($i, $tmp) = _binary_to_term_sequence($i, $length, $data);
             list($i, $tail) = _binary_to_term($i, $data);
             if (get_class($tail) != 'Erlang\OtpErlangList' or
                 $tail->value != array())
@@ -514,9 +537,10 @@ function _binary_to_term($i, $data)
             $i += 1;
             return array($i + $j, $bignum);
         case TAG_NEW_FUN_EXT:
-            list(, $size) = unpack('N', substr($data, $i, 4));
-            return array($i + $size,
-                         new OtpErlangFunction($tag, substr($data, $i, $size)));
+            list(, $length) = unpack('N', substr($data, $i, 4));
+            return array($i + $length,
+                         new OtpErlangFunction($tag, substr($data, $i,
+                                                            $length)));
         case TAG_EXPORT_EXT:
             $old_i = $i;
             list($i, $module) = _binary_to_atom($i, $data);
@@ -552,12 +576,12 @@ function _binary_to_term($i, $data)
                 $tmp = new OtpErlangAtom($atom_name);
             return array($i + $j, $tmp);
         case TAG_MAP_EXT:
-            list(, $arity) = unpack('N', substr($data, $i, 4));
+            list(, $length) = unpack('N', substr($data, $i, 4));
             $i += 4;
             $pairs = array();
-            if ($arity > 0)
+            if ($length > 0)
             {
-                foreach (range(0, $arity - 1) as $arity_index)
+                foreach (range(0, $length - 1) as $length_index)
                 {
                     list($i, $key) = _binary_to_term($i, $data);
                     list($i, $value) = _binary_to_term($i, $data);
@@ -607,12 +631,12 @@ function _binary_to_term($i, $data)
     }
 }
 
-function _binary_to_term_sequence($i, $arity, $data)
+function _binary_to_term_sequence($i, $length, $data)
 {
     $sequence = array();
-    if ($arity > 0)
+    if ($length > 0)
     {
-        foreach (range(0, $arity - 1) as $arity_index)
+        foreach (range(0, $length - 1) as $length_index)
         {
             list($i, $element) = _binary_to_term($i, $data);
             $sequence[] = $element;
@@ -621,6 +645,8 @@ function _binary_to_term_sequence($i, $arity, $data)
     return array($i, $sequence);
 }
 
+// (binary_to_term Erlang term primitive type functions)
+
 function _binary_to_integer($i, $data)
 {
     $tag = ord($data[$i]);
@@ -694,6 +720,8 @@ function _binary_to_atom($i, $data)
     }
 }
 
+// term_to_binary implementation functions
+
 function _term_to_binary($term)
 {
     if (is_string($term))
@@ -712,6 +740,11 @@ function _term_to_binary($term)
             $object = new OtpErlangAtom('false');
         return $object->binary();
     }
+    elseif (is_null($term))
+    {
+        $object = new OtpErlangAtom('undefined');
+        return $object->binary();
+    }
     elseif (is_object($term))
     {
         switch (get_class($term))
@@ -735,43 +768,53 @@ function _term_to_binary($term)
     }
 }
 
+// (term_to_binary Erlang term composite type functions)
+
 function _string_to_binary($term)
 {
-    $arity = strlen($term);
-    if ($arity == 0)
+    $length = strlen($term);
+    if ($length == 0)
     {
         return chr(TAG_NIL_EXT);
     }
-    elseif ($arity < 65536)
+    elseif ($length <= 65535)
     {
-        return pack('Cn', TAG_STRING_EXT, $arity) . $term;
+        return pack('Cn', TAG_STRING_EXT, $length) . $term;
     }
-    else
+    elseif ($length <= 4294967295)
     {
         $term_packed = '';
         foreach (str_split($term) as $c)
         {
             $term_packed .= chr(TAG_SMALL_INTEGER_EXT) . $c;
         }
-        return pack('CN', TAG_LIST_EXT, $arity) . $term_packed .
+        return pack('CN', TAG_LIST_EXT, $length) . $term_packed .
                chr(TAG_NIL_EXT);
     }
+    else
+    {
+        throw new OutputException('uint32 overflow');
+    }
 }
 
 function _tuple_to_binary($term)
 {
-    $arity = count($term);
+    $length = count($term);
     $term_packed = '';
     foreach ($term as $element)
     {
         $term_packed .= _term_to_binary($element);
     }
-    if ($arity < 256)
-        return pack('CC', TAG_SMALL_TUPLE_EXT, $arity) . $term_packed;
+    if ($length <= 255)
+        return pack('CC', TAG_SMALL_TUPLE_EXT, $length) . $term_packed;
+    elseif ($length <= 4294967295)
+        return pack('CN', TAG_LARGE_TUPLE_EXT, $length) . $term_packed;
     else
-        return pack('CN', TAG_LARGE_TUPLE_EXT, $arity) . $term_packed;
+        throw new OutputException('uint32 overflow');
 }
 
+// (term_to_binary Erlang term primitive type functions)
+
 function _integer_to_binary($term)
 {
     if (0 <= $term and $term <= 255)
@@ -786,29 +829,23 @@ function _bignum_to_binary($term)
 {
     // in PHP only for supporting integers > 32 bits (no native bignums)
     $bignum = abs($term);
-    $size = intval(ceil(_bignum_bit_length($bignum) / 8.0));
     if ($term < 0)
-        $sign = chr(1);
+        $sign = 1;
     else
-        $sign = chr(0);
-    $l = $sign;
-    if ($size > 0)
+        $sign = 0;
+    $l = '';
+    while ($bignum > 0)
     {
-        foreach (range(0, $size - 1) as $byte)
-        {
-            $l .= chr($bignum & 255);
-            $bignum >>= 8;
-        }
+        $l .= chr($bignum & 255);
+        $bignum >>= 8;
     }
-    if ($size < 256)
-        return pack('CC', TAG_SMALL_BIG_EXT, $size) . $l;
+    $length = strlen($l);
+    if ($length <= 255)
+        return pack('CCC', TAG_SMALL_BIG_EXT, $length, $sign) . $l;
+    elseif ($length <= 4294967295)
+        return pack('CNC', TAG_LARGE_BIG_EXT, $length, $sign) . $l;
     else
-        return pack('CN', TAG_LARGE_BIG_EXT, $size) . $l;
-}
-
-function _bignum_bit_length($bignum)
-{
-    return strlen(decbin($bignum));
+        throw new OutputException('uint32 overflow');
 }
 
 function _float_to_binary($term)
@@ -819,13 +856,7 @@ function _float_to_binary($term)
         return chr(TAG_NEW_FLOAT_EXT) . pack('d', $term);
 }
 
-class ParseException extends \Exception
-{
-    public function __construct($message, $code = 0, Exception $previous = null)
-    {
-        parent::__construct($message, $code, $previous);
-    }
-}
+// Exception classes listed alphabetically
 
 class InputException extends \Exception
 {
@@ -843,4 +874,12 @@ class OutputException extends \Exception
     }
 }
 
+class ParseException extends \Exception
+{
+    public function __construct($message, $code = 0, Exception $previous = null)
+    {
+        parent::__construct($message, $code, $previous);
+    }
+}
+
 ?>
