@@ -25,12 +25,55 @@
 
 -module(cloudi_x_bear).
 
--compile([export_all]).
-
--export([
-         get_statistics/1,
-         get_statistics/2
-        ]).
+-export([get_statistics/1,
+         get_statistics_subset/2,
+         get_null_statistics_subset/2,
+         calc_steps/1,
+         level/1,
+         report_subset/5,
+         get_statistics/2,
+         scan_values/1,
+         scan_values/2,
+         scan_values2/2,
+         scan_values2/3,
+         arithmetic_mean/1,
+         geometric_mean/1,
+         harmonic_mean/1,
+         percentile/3,
+         variance/2,
+         std_deviation/2,
+         skewness/2,
+         kurtosis/2,
+         get_histogram/3,
+         update_bin/3,
+         get_covariance/2,
+         get_kendall_correlation/2,
+         get_spearman_correlation/2,
+         ranks_of/1,
+         ranks_of/5,
+         get_pearson_correlation/2,
+         revsort/1,
+         foldl2/4,
+         math_log/1,
+         inverse/1,
+         get_hist_bins/4,
+         get_bin_list/3,
+         round_bin/1,
+         round_bin/2,
+         get_bin_width/2,
+         get_bin_count/3,
+         kendall_correlation/2,
+         simple_ranking/1,
+         tied_ranking/1,
+         tied_ordered_ranking/1,
+         tied_ordered_ranking/3,
+         kendall_right_of/2,
+         kendall_right_of_item/2,
+         tied_add_prev/2,
+         tied_rank_worker/3,
+         percentiles/3,
+         pick_items/3,
+         perc/2]).
 
 -define(HIST_BINS, 10).
 
@@ -369,15 +412,15 @@ foldl2(F, Acc, [I1|L1], [I2|L2]) when is_function(F,3) ->
 foldl2(_F, Acc, [], []) ->
     Acc.
 
-%% wrapper for math:cloudi_x_log/1 to avoid dividing by zero
+%% wrapper for math:log/1 to avoid dividing by zero
 math_log(0) ->
     1;
 math_log(0.0) ->
     1.0;
 math_log(X) when X < 0 ->
-    0; % it's not possible to take a cloudi_x_log of a negative number, return 0
+    0; % it's not possible to take a log of a negative number, return 0
 math_log(X) ->
-    math:cloudi_x_log(X).
+    math:log(X).
 
 %% wrapper for calculating inverse to avoid dividing by zero
 inverse(0) ->
