@@ -6,45 +6,31 @@
 %%% ==cloudi_x_cgroups Manipulation Functions==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2016-2018 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @copyright 2016-2018 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_cgroups).
@@ -76,6 +62,12 @@
                         {path_mounts, string() | undefined}).
 -export_type([options/0]).
 
+% for features specific to Erlang/OTP version 20.x (and later versions)
+-ifdef(ERLANG_OTP_VERSION_19).
+-else.
+-define(ERLANG_OTP_VERSION_20_FEATURES, true).
+-endif.
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -402,7 +394,7 @@ new_paths(PathV1, PathV2, undefined) ->
 new_paths(PathV1, PathV2, PathMounts) ->
     case shell("cat \"~s\"", [PathMounts]) of
         {0, Mounts} ->
-            MountsL = string:tokens(shell_output_string(Mounts), "\n"),
+            MountsL = split(shell_output_string(Mounts), "\n"),
             {MountsPathV1,
              MountsPathV2} = new_mounts(MountsL),
             ResultV1 = if
@@ -428,7 +420,7 @@ new_mounts(MountsL) ->
 new_mounts([], PathV1, PathV2) ->
     {PathV1, PathV2};
 new_mounts([Mount | MountsL], PathV1, PathV2) ->
-    case string:tokens(Mount, " ") of
+    case split(Mount, " ") of
         [_, NewPathV1, "cgroup" | _] ->
             new_mounts(MountsL, NewPathV1 ++ "/", PathV2);
         [_, NewPathV2, "cgroup2" | _] ->
@@ -581,12 +573,12 @@ update_parameters(CGroupParameters, 2, CGroupPathFull, Path) ->
     ControlRemoved = ["-" ++ Controller || Controller <- Controllers],
     CGroupSubPathFull = subdirectory(CGroupPathFull),
     case subtree_control_add(CGroupSubPathFull,
-                             string:join(ControlAdded, " "),
+                             lists:join($ , ControlAdded),
                              Path) of
         ok ->
             Result = update_parameters(CGroupParameters, CGroupPathFull),
             case subtree_control_remove(CGroupSubPathFull,
-                                        string:join(ControlRemoved, " "),
+                                        lists:join($ , ControlRemoved),
                                         Path) of
                 ok ->
                     Result;
@@ -709,6 +701,27 @@ strip(Value) ->
         not (C == $  orelse C == $\t orelse C == $\n orelse C == $\r)
     end, Value).
 
+-ifdef(ERLANG_OTP_VERSION_20_FEATURES).
+split(String, SearchPattern) ->
+    string:split(String, SearchPattern, all).
+-else.
+split(String, SearchPattern)
+    when is_list(String) ->
+    [erlang:binary_to_list(S)
+     || S <- split(erlang:list_to_binary(String), SearchPattern)];
+split(String, SearchPattern)
+    when is_binary(String) ->
+    Pattern = if
+        is_binary(SearchPattern) ->
+            [SearchPattern];
+        is_integer(hd(SearchPattern)) ->
+            [erlang:list_to_binary(SearchPattern)];
+        is_list(SearchPattern) ->
+            [erlang:iolist_to_binary(S) || S <- SearchPattern]
+    end,
+    binary:split(String, Pattern, [global]).
+-endif.
+
 cgroup_path_valid([]) ->
     true;
 cgroup_path_valid([_ | _] = CGroupPath) ->
