@@ -8,45 +8,31 @@
 %%% "setcap 'CAP_SYS_RESOURCE=+ep' executable" (on Linux).
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @copyright 2015-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_os_process).
@@ -61,6 +47,7 @@
          cgroup_format/2,
          cgroup_set/2,
          cgroup_unset/2,
+         chroot_format/2,
          directory_format/2]).
 
 -include("cloudi_core_i_constants.hrl").
@@ -436,6 +423,24 @@ cgroup_unset(OSPid, Values)
     ok = cloudi_x_cgroups:destroy(CGroups),
     ok.
 
+-spec chroot_format(Chroot :: cloudi_service_api:chroot_external(),
+                    EnvironmentLookup :: cloudi_environment:lookup()) ->
+    {ok, NewChroot :: string()} |
+    {error, any()}.
+
+chroot_format(undefined, _) ->
+    {ok, ""};
+chroot_format(Chroot, EnvironmentLookup) ->
+    NewChroot = cloudi_environment:transform(Chroot, EnvironmentLookup),
+    Valid = absolute_path(NewChroot) andalso
+            filelib:is_dir(NewChroot),
+    if
+        Valid =:= true ->
+            {ok, NewChroot};
+        Valid =:= false ->
+            {error, {service_options_chroot_invalid, NewChroot}}
+    end.
+
 -spec directory_format(Directory :: cloudi_service_api:directory_external(),
                        EnvironmentLookup :: cloudi_environment:lookup()) ->
     {ok, NewDirectory :: string()} |
@@ -445,10 +450,22 @@ directory_format(undefined, _) ->
     {ok, ""};
 directory_format(Directory, EnvironmentLookup) ->
     NewDirectory = cloudi_environment:transform(Directory, EnvironmentLookup),
-    case filelib:is_dir(NewDirectory) of
-        true ->
+    Valid = absolute_path(NewDirectory),
+    if
+        Valid =:= true ->
             {ok, NewDirectory};
-        false ->
+        Valid =:= false ->
             {error, {service_options_directory_invalid, NewDirectory}}
     end.
 
+%%%------------------------------------------------------------------------
+%%% Private functions
+%%%------------------------------------------------------------------------
+
+absolute_path([]) ->
+    false;
+absolute_path([_ | _] = Path) ->
+    [H | _] = AbsolutePath = filename:absname(Path),
+    (AbsolutePath == Path) orelse
+    (AbsolutePath ++ [H] == Path).
+
