@@ -5,45 +5,31 @@
 %%% ==CloudI Service Filesystem Tests==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2014-2015, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2015 Michael Truog
-%%% @version 1.5.0 {@date} {@time}
+%%% @copyright 2014-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_filesystem_SUITE).
@@ -77,13 +63,7 @@
 -define(SERVICE_PREFIX1, "/filesystem/").
 % ASCII_FILE created with: file:write_file("ASCII.bin", lists:seq(0, 126)).
 -define(ASCII_FILE, "ASCII.bin").
--define(WRITABLE_DIRECTORY, "/tmp/").
--define(WRITABLE_DIRECTORY_PRIVATE, "/tmp/cloudi_service_filesystem_test/").
--define(WRITABLE_FILENAME, "cloudi_service_filesystem_test.txt").
--define(WRITABLE_FILEPATH,
-        ?WRITABLE_DIRECTORY ?WRITABLE_FILENAME).
--define(WRITABLE_FILEPATH_PRIVATE,
-        ?WRITABLE_DIRECTORY_PRIVATE ?WRITABLE_FILENAME).
+-define(WRITABLE_FILENAME1, "filename1.txt").
 -define(TIMEOUT, 960000).
 -define(REFRESH, 2400). % ((?TIMEOUT * 2.5) div 1000)
 -define(REFRESH_STRING, "2400").
@@ -113,12 +93,31 @@ suite() ->
 
 init_per_suite(Config) ->
     ?REFRESH = erlang:round(?TIMEOUT * 2.5) div 1000,
-    ok = filelib:ensure_dir(?WRITABLE_FILEPATH_PRIVATE),
     ok = cloudi_x_reltool_util:application_start(cloudi_core, [], infinity),
-    Config.
+    TmpDir = os:getenv("TMPDIR", "/tmp"),
+    TmpDir1 = TmpDir ++ "/cloudi_service_filesystem_test_dir1/",
+    TmpDir2 = TmpDir ++ "/cloudi_service_filesystem_test_dir2/",
+    ok = filelib:ensure_dir(TmpDir1),
+    ok = filelib:ensure_dir(TmpDir2),
+    [{tmpdir1, TmpDir1},
+     {tmpdir2, TmpDir2} | Config].
 
-end_per_suite(_Config) ->
+end_per_suite(Config) ->
     ok = cloudi_x_reltool_util:application_stop(cloudi_core),
+    TmpDir1Rm = os:cmd("rmdir " ++ ?config(tmpdir1, Config)),
+    if
+        TmpDir1Rm == [] ->
+            ok;
+        true ->
+            ?LOG_ERROR("~s", [TmpDir1Rm])
+    end,
+    TmpDir2Rm = os:cmd("rmdir " ++ ?config(tmpdir2, Config)),
+    if
+        TmpDir2Rm == [] ->
+            ok;
+        true ->
+            ?LOG_ERROR("~s", [TmpDir2Rm])
+    end,
     ok.
 
 group(_GroupName) ->
@@ -162,15 +161,15 @@ init_per_testcase(TestCase, Config)
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
     when (TestCase =:= t_filesystem_basic_write_truncate_1) ->
-    ok = file:write_file(?WRITABLE_FILEPATH, <<>>),
+    ok = file:write_file(writable_filepath1(Config), <<>>),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_filesystem},
          {args,
-          [{directory, ?WRITABLE_DIRECTORY},
+          [{directory, ?config(tmpdir1, Config)},
            {write_truncate,
-            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME]},
+            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME1]},
            {use_content_disposition, true}]},
          {dest_refresh, none},
          {timeout_init, ?TIMEOUT},
@@ -179,17 +178,17 @@ init_per_testcase(TestCase, Config)
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
     when (TestCase =:= t_filesystem_basic_write_truncate_wcache_1) ->
-    ok = file:write_file(?WRITABLE_FILEPATH, <<>>),
+    ok = file:write_file(writable_filepath1(Config), <<>>),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_filesystem},
          {args,
-          [{directory, ?WRITABLE_DIRECTORY},
+          [{directory, ?config(tmpdir1, Config)},
            {cache, 1},
            {refresh, ?REFRESH},
            {write_truncate,
-            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME]},
+            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME1]},
            {use_content_disposition, true}]},
          {dest_refresh, none},
          {timeout_init, ?TIMEOUT},
@@ -198,15 +197,15 @@ init_per_testcase(TestCase, Config)
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
     when (TestCase =:= t_filesystem_basic_write_append_1) ->
-    ok = file:write_file(?WRITABLE_FILEPATH, <<>>),
+    ok = file:write_file(writable_filepath1(Config), <<>>),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_filesystem},
          {args,
-          [{directory, ?WRITABLE_DIRECTORY},
+          [{directory, ?config(tmpdir1, Config)},
            {write_append,
-            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME]},
+            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME1]},
            {use_content_disposition, true}]},
          {dest_refresh, none},
          {timeout_init, ?TIMEOUT},
@@ -215,17 +214,17 @@ init_per_testcase(TestCase, Config)
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
     when (TestCase =:= t_filesystem_basic_write_append_wcache_1) ->
-    ok = file:write_file(?WRITABLE_FILEPATH, <<>>),
+    ok = file:write_file(writable_filepath1(Config), <<>>),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_filesystem},
          {args,
-          [{directory, ?WRITABLE_DIRECTORY},
+          [{directory, ?config(tmpdir1, Config)},
            {cache, 1},
            {refresh, ?REFRESH},
            {write_append,
-            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME]},
+            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME1]},
            {use_content_disposition, true}]},
          {dest_refresh, none},
          {timeout_init, ?TIMEOUT},
@@ -250,16 +249,16 @@ init_per_testcase(TestCase, Config)
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
     when (TestCase =:= t_filesystem_size_limit_1) ->
-    file:delete(?WRITABLE_FILEPATH_PRIVATE),
+    _ = file:delete(writable_filepath2(Config)),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_filesystem},
          {args,
-          [{directory, ?WRITABLE_DIRECTORY_PRIVATE},
+          [{directory, ?config(tmpdir2, Config)},
            {files_size, 2}, % Kb
            {write_append,
-            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME]},
+            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME1]},
            {use_content_disposition, true}]},
          {dest_refresh, none},
          {timeout_init, ?TIMEOUT},
@@ -268,16 +267,16 @@ init_per_testcase(TestCase, Config)
     [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config)
     when (TestCase =:= t_filesystem_size_limit_2) ->
-    file:delete(?WRITABLE_FILEPATH_PRIVATE),
+    _ = file:delete(writable_filepath2(Config)),
     {ok, ServiceIds} = cloudi_service_api:services_add([
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_filesystem},
          {args,
-          [{directory, ?WRITABLE_DIRECTORY_PRIVATE},
+          [{directory, ?config(tmpdir2, Config)},
            {files_size, 1}, % Kb
            {write_truncate,
-            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME]},
+            [?SERVICE_PREFIX1 ?WRITABLE_FILENAME1]},
            {use_content_disposition, true}]},
          {dest_refresh, none},
          {timeout_init, ?TIMEOUT},
@@ -293,10 +292,10 @@ end_per_testcase(TestCase, Config) ->
         TestCase =:= t_filesystem_basic_write_truncate_wcache_1;
         TestCase =:= t_filesystem_basic_write_append_1;
         TestCase =:= t_filesystem_basic_write_append_wcache_1 ->
-            ok = file:delete(?WRITABLE_FILEPATH);
+            ok = file:delete(writable_filepath1(Config));
         TestCase =:= t_filesystem_size_limit_1;
         TestCase =:= t_filesystem_size_limit_2 ->
-            ok = file:delete(?WRITABLE_FILEPATH_PRIVATE);
+            ok = file:delete(writable_filepath2(Config));
         true ->
             ok
     end,
@@ -704,17 +703,17 @@ t_filesystem_basic_read_wcache_1(_Config) ->
                           Timeout, Priority),
     ok.
 
-t_filesystem_basic_write_truncate_1(_Config) ->
+t_filesystem_basic_write_truncate_1(Config) ->
     Context0 = cloudi:new(),
-    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/get",
-    ServiceNamePut = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/put",
+    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/get",
+    ServiceNamePut = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/put",
     Timeout = undefined, % default
     Priority = undefined, % default
-    {ok, <<>>} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, <<>>} = file:read_file(writable_filepath1(Config)),
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag0},
        {<<"last-modified">>, LastModified0},
        {<<"date">>, _},
@@ -727,7 +726,7 @@ t_filesystem_basic_write_truncate_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag1},
        {<<"last-modified">>, LastModified1},
        {<<"date">>, _},
@@ -740,7 +739,7 @@ t_filesystem_basic_write_truncate_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag2},
        {<<"last-modified">>, LastModified2},
        {<<"date">>, _},
@@ -753,7 +752,7 @@ t_filesystem_basic_write_truncate_1(_Config) ->
     true = LastModified0 =< LastModified1,
     true = ETag1 /= ETag2,
     true = LastModified1 =< LastModified2,
-    {ok, Response2} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, Response2} = file:read_file(writable_filepath1(Config)),
     {{ok,
       [{<<"status">>, <<"400">>}],
       <<>>},
@@ -762,17 +761,17 @@ t_filesystem_basic_write_truncate_1(_Config) ->
                            Timeout, Priority),
     ok.
 
-t_filesystem_basic_write_truncate_wcache_1(_Config) ->
+t_filesystem_basic_write_truncate_wcache_1(Config) ->
     Context0 = cloudi:new(),
-    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/get",
-    ServiceNamePut = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/put",
+    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/get",
+    ServiceNamePut = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/put",
     Timeout = undefined, % default
     Priority = undefined, % default
-    {ok, <<>>} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, <<>>} = file:read_file(writable_filepath1(Config)),
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag0},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -787,7 +786,7 @@ t_filesystem_basic_write_truncate_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag1},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -802,7 +801,7 @@ t_filesystem_basic_write_truncate_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag2},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -817,7 +816,7 @@ t_filesystem_basic_write_truncate_wcache_1(_Config) ->
     true = LastModified0 =< LastModified1,
     true = ETag1 /= ETag2,
     true = LastModified1 =< LastModified2,
-    {ok, Response2} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, Response2} = file:read_file(writable_filepath1(Config)),
     {{ok,
       [{<<"status">>, <<"400">>}],
       <<>>},
@@ -826,17 +825,17 @@ t_filesystem_basic_write_truncate_wcache_1(_Config) ->
                            Timeout, Priority),
     ok.
 
-t_filesystem_basic_write_append_1(_Config) ->
+t_filesystem_basic_write_append_1(Config) ->
     Context0 = cloudi:new(),
-    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/get",
-    ServiceNamePost = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/post",
+    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/get",
+    ServiceNamePost = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/post",
     Timeout = undefined, % default
     Priority = undefined, % default
-    {ok, <<>>} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, <<>>} = file:read_file(writable_filepath1(Config)),
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag0},
        {<<"last-modified">>, LastModified0},
        {<<"date">>, _},
@@ -849,7 +848,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag1},
        {<<"last-modified">>, LastModified1},
        {<<"date">>, _},
@@ -867,7 +866,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag2},
        {<<"last-modified">>, LastModified2},
        {<<"date">>, _},
@@ -888,7 +887,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag3},
        {<<"last-modified">>, LastModified3},
        {<<"date">>, _},
@@ -907,7 +906,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag4},
        {<<"last-modified">>, LastModified4},
        {<<"date">>, _},
@@ -926,7 +925,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag5},
        {<<"last-modified">>, LastModified5},
        {<<"date">>, _},
@@ -957,7 +956,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag7},
        {<<"last-modified">>, LastModified7},
        {<<"date">>, _},
@@ -974,7 +973,7 @@ t_filesystem_basic_write_append_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag8},
        {<<"last-modified">>, LastModified8},
        {<<"date">>, _},
@@ -985,20 +984,20 @@ t_filesystem_basic_write_append_1(_Config) ->
                            Timeout, Priority),
     true = ETag7 /= ETag8,
     true = LastModified7 =< LastModified8,
-    {ok, Response8} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, Response8} = file:read_file(writable_filepath1(Config)),
     ok.
 
-t_filesystem_basic_write_append_wcache_1(_Config) ->
+t_filesystem_basic_write_append_wcache_1(Config) ->
     Context0 = cloudi:new(),
-    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/get",
-    ServiceNamePost = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/post",
+    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/get",
+    ServiceNamePost = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/post",
     Timeout = undefined, % default
     Priority = undefined, % default
-    {ok, <<>>} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, <<>>} = file:read_file(writable_filepath1(Config)),
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag0},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1013,7 +1012,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag1},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1033,7 +1032,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag2},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1056,7 +1055,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag3},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1077,7 +1076,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag4},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1098,7 +1097,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag5},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1131,7 +1130,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag7},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1150,7 +1149,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag8},
        {<<"cache-control">>, <<"public,max-age=1">>},
        {<<"expires">>, _},
@@ -1163,7 +1162,7 @@ t_filesystem_basic_write_append_wcache_1(_Config) ->
                            Timeout, Priority),
     true = ETag7 /= ETag8,
     true = LastModified7 =< LastModified8,
-    {ok, Response8} = file:read_file(?WRITABLE_FILEPATH),
+    {ok, Response8} = file:read_file(writable_filepath1(Config)),
     ok.
 
 t_filesystem_basic_read_cache_1(_Config) ->
@@ -1291,16 +1290,16 @@ t_filesystem_basic_read_cache_1(_Config) ->
                            Timeout, Priority),
     ok.
 
-t_filesystem_size_limit_1(_Config) ->
+t_filesystem_size_limit_1(Config) ->
     Context0 = cloudi:new(),
-    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/get",
-    ServiceNamePost = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/post",
+    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/get",
+    ServiceNamePost = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/post",
     Timeout = undefined, % default
     Priority = undefined, % default
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag0},
        {<<"last-modified">>, LastModified0},
        {<<"date">>, _},
@@ -1309,12 +1308,12 @@ t_filesystem_size_limit_1(_Config) ->
      Context1} = cloudi:send_sync(Context0, ServiceNameGet,
                                   <<>>, <<>>,
                                   Timeout, Priority),
-    {error, enoent} = file:read_file(?WRITABLE_FILEPATH_PRIVATE),
+    {error, enoent} = file:read_file(writable_filepath2(Config)),
     Request1 = <<0:8192>>, % 1 Kb
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag1},
        {<<"last-modified">>, LastModified1},
        {<<"date">>, _},
@@ -1323,7 +1322,7 @@ t_filesystem_size_limit_1(_Config) ->
      Context2} = cloudi:send_sync(Context1, ServiceNamePost,
                                   <<>>, Request1,
                                   Timeout, Priority),
-    {ok, Request1} = file:read_file(?WRITABLE_FILEPATH_PRIVATE),
+    {ok, Request1} = file:read_file(writable_filepath2(Config)),
     true = ETag0 /= ETag1,
     true = LastModified0 =< LastModified1,
     Request2 = <<1:8192>>, % 1 Kb
@@ -1331,7 +1330,7 @@ t_filesystem_size_limit_1(_Config) ->
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag2},
        {<<"last-modified">>, LastModified2},
        {<<"date">>, _},
@@ -1342,7 +1341,7 @@ t_filesystem_size_limit_1(_Config) ->
                                   Timeout, Priority),
     true = ETag1 /= ETag2,
     true = LastModified1 =< LastModified2,
-    {ok, Response2} = file:read_file(?WRITABLE_FILEPATH_PRIVATE),
+    {ok, Response2} = file:read_file(writable_filepath2(Config)),
     Request3 = <<2:8>>, % 1 byte
     {{ok,
       [{<<"status">>, <<"400">>}],
@@ -1352,16 +1351,16 @@ t_filesystem_size_limit_1(_Config) ->
                            Timeout, Priority),
     ok.
 
-t_filesystem_size_limit_2(_Config) ->
+t_filesystem_size_limit_2(Config) ->
     Context0 = cloudi:new(),
-    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/get",
-    ServiceNamePut = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME ++ "/put",
+    ServiceNameGet = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/get",
+    ServiceNamePut = ?SERVICE_PREFIX1 ++ ?WRITABLE_FILENAME1 ++ "/put",
     Timeout = undefined, % default
     Priority = undefined, % default
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag0},
        {<<"last-modified">>, LastModified0},
        {<<"date">>, _},
@@ -1370,12 +1369,12 @@ t_filesystem_size_limit_2(_Config) ->
      Context1} = cloudi:send_sync(Context0, ServiceNameGet,
                                   <<>>, <<>>,
                                   Timeout, Priority),
-    {error, enoent} = file:read_file(?WRITABLE_FILEPATH_PRIVATE),
+    {error, enoent} = file:read_file(writable_filepath2(Config)),
     Request1 = <<0:8192>>, % 1 Kb
     {{ok,
       [{<<"content-type">>, <<"text/plain">>},
        {<<"content-disposition">>,
-        <<"attachment; filename=\"" ?WRITABLE_FILENAME "\"">>},
+        <<"attachment; filename=\"" ?WRITABLE_FILENAME1 "\"">>},
        {<<"etag">>, ETag1},
        {<<"last-modified">>, LastModified1},
        {<<"date">>, _},
@@ -1384,7 +1383,7 @@ t_filesystem_size_limit_2(_Config) ->
      Context2} = cloudi:send_sync(Context1, ServiceNamePut,
                                   <<>>, Request1,
                                   Timeout, Priority),
-    {ok, Request1} = file:read_file(?WRITABLE_FILEPATH_PRIVATE),
+    {ok, Request1} = file:read_file(writable_filepath2(Config)),
     true = ETag0 /= ETag1,
     true = LastModified0 =< LastModified1,
     Request2 = <<0:8200>>, % 1 Kb + 1 byte
@@ -1400,3 +1399,9 @@ t_filesystem_size_limit_2(_Config) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+writable_filepath1(Config) ->
+    filename:join(?config(tmpdir1, Config), ?WRITABLE_FILENAME1).
+
+writable_filepath2(Config) ->
+    filename:join(?config(tmpdir2, Config), ?WRITABLE_FILENAME1).
+
