@@ -6,45 +6,31 @@
 %%% ==Cowboy CloudI HTTP Handler==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2012-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2012-2018 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2012-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2012-2018 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_cowboy_handler).
@@ -71,8 +57,6 @@
 -include_lib("cloudi_core/include/cloudi_service_children.hrl").
 -include("cloudi_http_cowboy_handler.hrl").
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
-
 -record(websocket_state,
     {
         % for service requests entering CloudI
@@ -88,10 +72,9 @@
             :: undefined | cloudi:message_service_request(),
         response_lookup
             :: undefined |
-               dict_proxy(any(), {cloudi:message_service_request(),
-                                  reference()}),
+               #{any() := {cloudi:message_service_request(), reference()}},
         recv_timeouts
-            :: undefined | dict_proxy(cloudi:trans_id(), reference()),
+            :: undefined | #{cloudi:trans_id() := reference()},
         queued
             :: undefined |
                cloudi_x_pqueue4:cloudi_x_pqueue4(
@@ -225,7 +208,7 @@ handle(Req0,
                 true ->
                     % handle custom methods, if they occur
                     NameIncoming ++ [$/ |
-                        string:to_lower(erlang:binary_to_list(Method))]
+                        cloudi_string:lowercase(erlang:binary_to_list(Method))]
             end,
             PeerShort = erlang:list_to_binary(inet_parse:ntoa(ClientIpAddr)),
             PeerLong = cloudi_ip_address:to_binary(ClientIpAddr),
@@ -409,13 +392,13 @@ websocket_init(_Transport, Req0,
     end,
     ResponseLookup = if
         WebSocketProtocol /= undefined ->
-            dict:new();
+            #{};
         true ->
             undefined
     end,
     RecvTimeouts = if
         WebSocketProtocol =:= undefined ->
-            dict:new();
+            #{};
         true ->
             undefined
     end,
@@ -587,7 +570,7 @@ websocket_handle({WebSocketRequestType, RequestBinary}, Req,
         {incoming, Request} ->
             {undefined, undefined, Request};
         {ID, Response} ->
-            case dict:find(ID, ResponseLookup) of
+            case maps:find(ID, ResponseLookup) of
                 {ok, ResponseData} ->
                     {ID, ResponseData, Response};
                 error ->
@@ -616,7 +599,7 @@ websocket_handle({WebSocketRequestType, RequestBinary}, Req,
             {ok, Req,
              State#cowboy_state{websocket_state =
                  WebSocketState#websocket_state{
-                     response_lookup = dict:erase(LookupID, ResponseLookup)}
+                     response_lookup = maps:remove(LookupID, ResponseLookup)}
                  }}
     end.
 
@@ -630,7 +613,7 @@ websocket_info({response_timeout, ID}, Req,
     {ok, Req,
      State#cowboy_state{websocket_state =
          WebSocketState#websocket_state{
-             response_lookup = dict:erase(ID, ResponseLookup)}
+             response_lookup = maps:remove(ID, ResponseLookup)}
          }};
 
 websocket_info(response_timeout, Req,
@@ -699,7 +682,7 @@ websocket_info({Type, Name, Pattern, RequestInfo, RequestProtocol,
          Timeout, Priority, TransId, Source},
     ResponseTimer = erlang:send_after(Timeout, self(),
                                       {response_timeout, ID}),
-    NewResponseLookup = dict:store(ID, {T, ResponseTimer}, ResponseLookup),
+    NewResponseLookup = maps:put(ID, {T, ResponseTimer}, ResponseLookup),
     NewState = State#cowboy_state{
         websocket_state = WebSocketState#websocket_state{
             response_lookup = NewResponseLookup}},
@@ -736,7 +719,7 @@ websocket_info({Type, _, _, _, Request,
     {ok, Req,
      State#cowboy_state{
          websocket_state = WebSocketState#websocket_state{
-             recv_timeouts = dict:store(TransId,
+             recv_timeouts = maps:put(TransId,
                  erlang:send_after(Timeout, self(),
                      {'cloudi_service_recv_timeout', Priority, TransId}),
                  RecvTimeouts),
@@ -773,7 +756,7 @@ websocket_info({'cloudi_service_recv_timeout', Priority, TransId}, Req,
     {ok, Req,
      State#cowboy_state{
          websocket_state = WebSocketState#websocket_state{
-             recv_timeouts = dict:erase(TransId, RecvTimeouts),
+             recv_timeouts = maps:remove(TransId, RecvTimeouts),
              queued = NewQueue}
          }};
 
@@ -959,18 +942,18 @@ get_query_string_external_text([{K, V} | L]) ->
     end.
 
 request_time_start() ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 request_time_end_success(HttpCode, Method, NameIncoming, NameOutgoing,
                          RequestStartMicroSec) ->
     ?LOG_TRACE("~w ~s ~s (to ~s) ~p ms",
                [HttpCode, Method, NameIncoming, NameOutgoing,
-                (cloudi_x_uuid:get_v1_time(os) -
+                (cloudi_timestamp:microseconds_monotonic() -
                  RequestStartMicroSec) / 1000.0]).
 
 request_time_end_error(HttpCode, Method, NameIncoming, NameOutgoing,
                        RequestStartMicroSec, Reason) ->
-    RequestTime = (cloudi_x_uuid:get_v1_time(os) -
+    RequestTime = (cloudi_timestamp:microseconds_monotonic() -
                    RequestStartMicroSec) / 1000.0,
     if
         NameOutgoing =:= undefined ->
@@ -984,20 +967,20 @@ request_time_end_error(HttpCode, Method, NameIncoming, NameOutgoing,
     end.
 
 websocket_time_start() ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 websocket_time_end_success(NameIncoming, NameOutgoing,
                            RequestStartMicroSec) ->
     ?LOG_TRACE("~s (to ~s) ~p ms",
                [NameIncoming, NameOutgoing,
-                (cloudi_x_uuid:get_v1_time(os) -
+                (cloudi_timestamp:microseconds_monotonic() -
                  RequestStartMicroSec) / 1000.0]).
 
 websocket_time_end_error(NameIncoming, NameOutgoing,
                          RequestStartMicroSec, Reason) ->
     ?LOG_WARN("~s (to ~s) ~p ms: ~p",
               [NameIncoming, NameOutgoing,
-               (cloudi_x_uuid:get_v1_time(os) -
+               (cloudi_timestamp:microseconds_monotonic() -
                 RequestStartMicroSec) / 1000.0, Reason]).
 
 websocket_request_end(Name, NewTimeout, OldTimeout) ->
@@ -1540,8 +1523,8 @@ websocket_process_queue(Req,
                  WebSocketState#websocket_state{queued = NewQueue}}};
         {{value, {Type, Name, Pattern, RequestInfo, Request,
                   _, Priority, TransId, Pid}}, NewQueue} ->
-            Timeout = case erlang:cancel_timer(dict:fetch(TransId,
-                                                          RecvTimeouts)) of
+            Timeout = case erlang:cancel_timer(maps:get(TransId,
+                                                        RecvTimeouts)) of
                 false ->
                     0;
                 V ->
@@ -1551,8 +1534,8 @@ websocket_process_queue(Req,
                             Timeout, Priority, TransId, Pid}, Req,
                            State#cowboy_state{websocket_state =
                                WebSocketState#websocket_state{
-                                   recv_timeouts = dict:erase(TransId,
-                                                              RecvTimeouts),
+                                   recv_timeouts = maps:remove(TransId,
+                                                               RecvTimeouts),
                                    queued = NewQueue}})
     end.
 
