@@ -6,45 +6,31 @@
 %%% ==Elli CloudI HTTP Handler==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2013-2015, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2013-2018 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2015 Michael Truog
-%%% @version 1.5.1 {@date} {@time}
+%%% @copyright 2013-2018 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_elli_handler).
@@ -179,8 +165,8 @@ handle(Req,
                 Method =:= 'OPTIONS' ->
                     NameIncoming ++ "/options";
                 is_binary(Method) ->
-                    NameIncoming ++
-                    [$/ | string:to_lower(erlang:binary_to_list(Method))]
+                    NameIncoming ++ [$/ |
+                        cloudi_string:lowercase(erlang:binary_to_list(Method))]
             end,
             PeerShort = erlang:list_to_binary(inet_parse:ntoa(ClientIpAddr)),
             PeerLong = cloudi_ip_address:to_binary(ClientIpAddr),
@@ -263,6 +249,8 @@ handle(Req,
 %% create ETS tables or start supervised processes in a supervisor
 %% tree.
 handle_event(elli_startup, [], _) -> ok;
+%% elli_reconfigure is an event after elli:set_callback/3 is called
+handle_event(elli_reconfigure, [], _) -> ok;
 %% request_complete fires *after* Elli has sent the response to the
 %% client. Timings contains timestamps of events like when the
 %% connection was accepted, when request parsing finished, when the
@@ -412,18 +400,18 @@ get_query_string_external_text([{K, V} | L]) ->
     end.
 
 request_time_start() ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 request_time_end_success(HttpCode, Method, NameIncoming, NameOutgoing,
                          RequestStartMicroSec) ->
     ?LOG_TRACE("~w ~s ~s (to ~s) ~p ms",
                [HttpCode, Method, NameIncoming, NameOutgoing,
-                (cloudi_x_uuid:get_v1_time(os) -
+                (cloudi_timestamp:microseconds_monotonic() -
                  RequestStartMicroSec) / 1000.0]).
 
 request_time_end_error(HttpCode, Method, NameIncoming, NameOutgoing,
                        RequestStartMicroSec, Reason) ->
-    RequestTime = (cloudi_x_uuid:get_v1_time(os) -
+    RequestTime = (cloudi_timestamp:microseconds_monotonic() -
                    RequestStartMicroSec) / 1000.0,
     if
         NameOutgoing =:= undefined ->
