@@ -9,45 +9,31 @@
 %%% arguments.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2015-2018 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_rest).
@@ -159,15 +145,13 @@
         arity :: 11
     }).
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
-
 -record(state,
     {
         prefix :: cloudi:service_name_pattern(),
         lookup :: cloudi_x_trie:cloudi_x_trie(),
         info_f :: info_f() | undefined,
         terminate_f :: terminate_f() | undefined,
-        content_types :: dict_proxy(atom(), {request | attachment, binary()}),
+        content_types :: #{atom() := {request | attachment, binary()}},
         content_disposition :: boolean(),
         debug_level :: off | trace | debug | info | warn | error | fatal,
         api_state :: any()
@@ -206,7 +190,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
             is_list(Method), is_integer(hd(Method)) ->
                 Method
         end,
-        MethodString = string:to_upper(MethodString),
+        MethodString = cloudi_string:uppercase(MethodString),
         true = is_list(Path) andalso is_integer(hd(Path))
     end, Handlers0),
     InfoN = cloudi_args_type:function_optional(Info0, 3),
@@ -239,10 +223,10 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         FormatN
     end, Formats0),
     ContentTypeLookupN = lists:foldl(fun(FormatN, ContentTypeLookup0) ->
-        dict:store(erlang:list_to_atom(FormatN),
-                   cloudi_x_trie:fetch("." ++ FormatN, ContentTypes),
-                   ContentTypeLookup0)
-    end, dict:new(), FormatsN),
+        maps:put(erlang:list_to_atom(FormatN),
+                 cloudi_x_trie:fetch("." ++ FormatN, ContentTypes),
+                 ContentTypeLookup0)
+    end, #{}, FormatsN),
     MethodListsN = cloudi_x_trie:to_list(lists:foldr(fun({Method, Path, _},
                                                          MethodLists0) ->
         MethodString = if
@@ -272,8 +256,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         UseOptionsMethod =:= true ->
             lists:map(fun({Path, MethodList1}) ->
                 MethodListN = lists:umerge(MethodList1, ["OPTIONS"]),
-                Methods = erlang:list_to_binary(string:join(MethodListN,
-                                                            ", ")),
+                Methods = erlang:list_to_binary(lists:join(", ", MethodListN)),
                 HandlerOptions = fun(_, _, _, _, _, _, _, _, _,
                                      OptionsHandlerState, _) ->
                     % content-type is set automatically by HTTP process
@@ -436,6 +419,8 @@ cloudi_service_handle_info(Request,
             end
     end.
 
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
+    ok;
 cloudi_service_terminate(Reason, Timeout,
                          #state{terminate_f = TerminateF,
                                 api_state = StateAPI}) ->
@@ -482,13 +467,13 @@ subscribe_paths('CONNECT', Path, Formats, API, Lookup, Dispatcher) ->
     subscribe_path(Formats, Path, "/connect", API, Lookup, Dispatcher);
 subscribe_paths(Method, Path, Formats, API, Lookup, Dispatcher)
     when is_list(Method) ->
-    MethodSuffix = [$/ | string:to_lower(Method)],
+    MethodSuffix = [$/ | cloudi_string:lowercase(Method)],
     subscribe_path(Formats, Path, MethodSuffix, API, Lookup, Dispatcher).
 
 response_info_headers(ResponseInfo0, Name, Format,
                       ContentTypes, ContentDisposition)
     when is_list(ResponseInfo0) ->
-    {AttachmentGuess, ContentType} = dict:fetch(Format, ContentTypes),
+    {AttachmentGuess, ContentType} = maps:get(Format, ContentTypes),
     ResponseInfo1 = if
         (ContentDisposition =:= true) andalso
         (AttachmentGuess =:= attachment) ->
