@@ -6,45 +6,31 @@
 %%% ==CloudI Monitoring Metrics==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @copyright 2015-2017 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring_cloudi).
@@ -70,6 +56,13 @@
          services_update/8,
          nodes_update/3]).
 
+%% internal functions used by anonymous aspect functions
+-export([aspect_init/0,
+         aspect_pid_to_service_id/1,
+         aspect_pid_to_object/0,
+         aspect_ref_to_object/2,
+         aspect_cloudi/0]).
+
 -include("cloudi_service_monitoring.hrl").
 -include("cloudi_service_monitoring_cloudi.hrl").
 
@@ -98,7 +91,7 @@
 
 -record(service_data,
     {
-        process_info :: dict_proxy(pid(), #process_info{}),
+        process_info :: #{pid() := #process_info{}},
         % modifications to ?ETS_PID2METRIC
         ets_insert = [] :: list(pid_object()),
         ets_delete = [] :: list(pid()),
@@ -107,7 +100,7 @@
         count_external = 0 :: non_neg_integer(),
         concurrency_internal = 0 :: non_neg_integer(),
         concurrency_external = 0 :: non_neg_integer(),
-        scopes = dict:new() :: dict_proxy(atom(), #scope_data{}),
+        scopes = #{} :: #{atom() := #scope_data{}},
         metrics = [] :: metric_list()
     }).
 
@@ -425,12 +418,89 @@ nodes_update(NodesVisible, NodesHidden, NodesAll) ->
      metric(gauge, [hidden], NodesHidden),
      metric(gauge, [all], NodesAll)].
 
+%%%------------------------------------------------------------------------
+%%% Internal functions used by anonymous aspect functions
+%%%------------------------------------------------------------------------
+
+aspect_init() ->
+    try ets:lookup(?ETS_CONFIG, init) of
+        [] ->
+            undefined;
+        [{init, MetricPrefix, Driver}] ->
+            Pid = self(),
+            case service_metric_id_from_pid(Pid) of
+                undefined ->
+                    undefined;
+                ServiceMetricId ->
+                    ServiceMetricPrefix = MetricPrefix ++
+                                          [services | ServiceMetricId],
+                    PidObject = {Pid, ServiceMetricPrefix, Driver},
+                    true = ets:insert(?ETS_PID2METRIC, PidObject),
+                    PidObject
+            end
+    catch
+        error:badarg ->
+            undefined
+    end.
+
+aspect_pid_to_service_id(Pid) ->
+    try ets:lookup(?ETS_PID2METRIC, Pid) of
+        [] ->
+            undefined;
+        [{_, MetricPrefix, _}] ->
+            service_metric_id_from_metric_prefix(MetricPrefix)
+    catch
+        error:badarg ->
+            undefined
+    end.
+
+aspect_pid_to_object() ->
+    try ets:lookup(?ETS_PID2METRIC, self()) of
+        [] ->
+            undefined;
+        [PidObject] ->
+            PidObject
+    catch
+        error:badarg ->
+            undefined
+    end.
+
+aspect_ref_to_object(Ref, Dispatcher)
+    when is_reference(Ref) ->
+    try ets:lookup(?ETS_REF2METRIC, Ref) of
+        [] ->
+            case ets:lookup(?ETS_PID2METRIC, cloudi_service:self(Dispatcher)) of
+                [] ->
+                    undefined;
+                [PidObject] ->
+                    RefObject = {Ref, PidObject},
+                    true = ets:insert(?ETS_REF2METRIC, RefObject),
+                    PidObject
+            end;
+        [{Ref, PidObject}] ->
+            PidObject
+    catch
+        error:badarg ->
+            undefined
+    end.
+
+aspect_cloudi() ->
+    try ets:lookup(?ETS_CONFIG, init) of
+        [] ->
+            undefined;
+        [{_, _, _} = Init] ->
+            Init
+    catch
+        error:badarg ->
+            undefined
+    end.
+
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
 
 scopes_accumulate_internal(Scope, Concurrency, Scopes) ->
-    dict:update(Scope,
+    maps:update_with(Scope,
                 fun(#scope_data{count_internal = Count,
                                 concurrency_internal = ConcurrencySum} = V) ->
         V#scope_data{count_internal = Count + 1,
@@ -439,7 +509,7 @@ scopes_accumulate_internal(Scope, Concurrency, Scopes) ->
                      concurrency_internal = Concurrency}, Scopes).
 
 scopes_accumulate_external(Scope, Concurrency, Scopes) ->
-    dict:update(Scope,
+    maps:update_with(Scope,
                 fun(#scope_data{count_external = Count,
                                 concurrency_external = ConcurrencySum} = V) ->
         V#scope_data{count_external = Count + 1,
@@ -672,7 +742,7 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                         external, ProcessInfo0, Pid,
                         QueuedEmptySize, MetricPrefix) ->
     case service_state(Pid) of
-        {ok, {_, State}} -> % gen_fsm
+        {ok, {_, State}} -> % gen_statem
             40 = erlang:tuple_size(State),
             state = erlang:element(1, State),
             Outgoing = ?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
@@ -738,7 +808,7 @@ service_metrics_pid_external([Pid | Pids], Metrics, ProcessInfo0,
     {[_],
      #service{process_index = ProcessIndex}} =
         cloudi_x_key2value:fetch2(Pid, Services),
-    ThreadIndex = case dict:find(ProcessIndex, ThreadIndexLookup) of
+    ThreadIndex = case maps:find(ProcessIndex, ThreadIndexLookup) of
         {ok, ThreadIndexNext} ->
             ThreadIndexNext;
         error ->
@@ -754,8 +824,8 @@ service_metrics_pid_external([Pid | Pids], Metrics, ProcessInfo0,
                                              QueuedEmptySize,
                                              ThreadMetricPrefix),
     service_metrics_pid_external(Pids, MetricsNew ++ Metrics, ProcessInfoN,
-                                 dict:store(ProcessIndex,
-                                            ThreadIndex + 1, ThreadIndexLookup),
+                                 maps:put(ProcessIndex,
+                                          ThreadIndex + 1, ThreadIndexLookup),
                                  QueuedEmptySize,
                                  Services, MetricPrefix).
 
@@ -765,7 +835,7 @@ service_metrics_pid(internal, Pids, ProcessInfo,
                                  QueuedEmptySize, Services, MetricPrefix);
 service_metrics_pid(external, Pids, ProcessInfo,
                     QueuedEmptySize, Services, MetricPrefix) ->
-    service_metrics_pid_external(Pids, [], ProcessInfo, dict:new(),
+    service_metrics_pid_external(Pids, [], ProcessInfo, #{},
                                  QueuedEmptySize, Services, MetricPrefix).
 
 service_metrics(Pids, ProcessInfo0,
@@ -792,7 +862,7 @@ service_state(Pid) ->
 
 services_metrics(CountInternal, CountExternal,
                  ConcurrencyInternal, ConcurrencyExternal, Scopes) ->
-    dict:fold(fun(Scope, 
+    maps:fold(fun(Scope, 
                   #scope_data{
                       count_internal = ScopeCountInternal,
                       count_external = ScopeCountExternal,
@@ -814,7 +884,7 @@ services_metrics(CountInternal, CountExternal,
                 ScopeCountExternal) | ScopeMetrics]
     end,
     [metric(gauge, [scopes, count],
-            dict:size(Scopes)),
+            maps:size(Scopes)),
      metric(gauge, [concurrency],
             ConcurrencyInternal + ConcurrencyExternal),
      metric(gauge, [count],
@@ -834,7 +904,7 @@ process_info_store(Pid, ProcessInfo) ->
         [{memory, MemoryNew},
          {message_queue_len, MessagesNew},
          {reductions, ReductionsNew}] ->
-            case dict:find(Pid, ProcessInfo) of
+            case maps:find(Pid, ProcessInfo) of
                 {ok, #process_info{reductions = ReductionsOld}} ->
                     ReductionsNow = if
                         ReductionsOld =:= undefined ->
@@ -846,15 +916,15 @@ process_info_store(Pid, ProcessInfo) ->
                                             message_queue_len = MessagesNew,
                                             reductions = ReductionsNew,
                                             reductions_now = ReductionsNow},
-                    dict:store(Pid, InfoNew, ProcessInfo);
+                    maps:put(Pid, InfoNew, ProcessInfo);
                 error ->
                     InfoNew = #process_info{memory = MemoryNew,
                                             message_queue_len = MessagesNew,
                                             reductions = ReductionsNew},
-                    dict:store(Pid, InfoNew, ProcessInfo)
+                    maps:put(Pid, InfoNew, ProcessInfo)
             end;
         undefined ->
-            dict:store(Pid, #process_info{}, ProcessInfo)
+            maps:put(Pid, #process_info{}, ProcessInfo)
     end.
 
 process_info_update(Pid, ProcessInfo) ->
@@ -862,7 +932,7 @@ process_info_update(Pid, ProcessInfo) ->
         [{memory, MemoryNew},
          {message_queue_len, MessagesNew},
          {reductions, ReductionsNew}] ->
-            case dict:find(Pid, ProcessInfo) of
+            case maps:find(Pid, ProcessInfo) of
                 {ok, #process_info{reductions = ReductionsOld}} ->
                     ReductionsNow = if
                         ReductionsOld =:= undefined ->
@@ -875,21 +945,21 @@ process_info_update(Pid, ProcessInfo) ->
                                             reductions = ReductionsNew,
                                             reductions_now = ReductionsNow},
                     {{MemoryNew, MessagesNew, ReductionsNow},
-                     dict:store(Pid, InfoNew, ProcessInfo)};
+                     maps:put(Pid, InfoNew, ProcessInfo)};
                 error ->
                     InfoNew = #process_info{memory = MemoryNew,
                                             message_queue_len = MessagesNew,
                                             reductions = ReductionsNew},
                     {{MemoryNew, MessagesNew, undefined},
-                     dict:store(Pid, InfoNew, ProcessInfo)}
+                     maps:put(Pid, InfoNew, ProcessInfo)}
             end;
         undefined ->
             {{undefined, undefined, undefined},
-             dict:store(Pid, #process_info{}, ProcessInfo)}
+             maps:put(Pid, #process_info{}, ProcessInfo)}
     end.
 
 process_info_find(Pid, ProcessInfo) ->
-    case dict:find(Pid, ProcessInfo) of
+    case maps:find(Pid, ProcessInfo) of
         {ok, #process_info{memory = Memory,
                            message_queue_len = Messages,
                            reductions_now = ReductionsNow}} ->
@@ -899,7 +969,7 @@ process_info_find(Pid, ProcessInfo) ->
     end.
 
 process_info_erase(Pid, ProcessInfo) ->
-    dict:erase(Pid, ProcessInfo).
+    maps:remove(Pid, ProcessInfo).
 
 process_info_metrics({Memory, Messages, ReductionsNow}, MetricPrefix) ->
     L0 = [],
@@ -936,83 +1006,12 @@ metric(spiral, [_ | _] = Name, Value) ->
 metric(gauge, [_ | _] = Name, Value) ->
     {gauge, Name, Value}.
 
-aspect_init() ->
-    try ets:lookup(?ETS_CONFIG, init) of
-        [] ->
-            undefined;
-        [{init, MetricPrefix, Driver}] ->
-            Pid = self(),
-            case service_metric_id_from_pid(Pid) of
-                undefined ->
-                    undefined;
-                ServiceMetricId ->
-                    ServiceMetricPrefix = MetricPrefix ++
-                                          [services | ServiceMetricId],
-                    PidObject = {Pid, ServiceMetricPrefix, Driver},
-                    true = ets:insert(?ETS_PID2METRIC, PidObject),
-                    PidObject
-            end
-    catch
-        error:badarg ->
-            undefined
-    end.
-
-aspect_pid_to_service_id(Pid) ->
-    try ets:lookup(?ETS_PID2METRIC, Pid) of
-        [] ->
-            undefined;
-        [{_, MetricPrefix, _}] ->
-            service_metric_id_from_metric_prefix(MetricPrefix)
-    catch
-        error:badarg ->
-            undefined
-    end.
-
-aspect_pid_to_object() ->
-    try ets:lookup(?ETS_PID2METRIC, self()) of
-        [] ->
-            undefined;
-        [PidObject] ->
-            PidObject
-    catch
-        error:badarg ->
-            undefined
-    end.
-
-aspect_ref_to_object(Ref, Dispatcher)
-    when is_reference(Ref) ->
-    try ets:lookup(?ETS_REF2METRIC, Ref) of
-        [] ->
-            case ets:lookup(?ETS_PID2METRIC, cloudi_service:self(Dispatcher)) of
-                [] ->
-                    undefined;
-                [PidObject] ->
-                    RefObject = {Ref, PidObject},
-                    true = ets:insert(?ETS_REF2METRIC, RefObject),
-                    PidObject
-            end;
-        [{Ref, PidObject}] ->
-            PidObject
-    catch
-        error:badarg ->
-            undefined
-    end.
-
-aspect_cloudi() ->
-    try ets:lookup(?ETS_CONFIG, init) of
-        [] ->
-            undefined;
-        [{_, _, _} = Init] ->
-            Init
-    catch
-        error:badarg ->
-            undefined
-    end.
-
 aspect_init_after_internal_f() ->
     fun(_Args, _Prefix, _Timeout, State, _Dispatcher) ->
-        case aspect_init() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_init() of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(spiral, MetricPrefix ++ [init], 1, Driver);
             undefined ->
                 ok
@@ -1022,8 +1021,10 @@ aspect_init_after_internal_f() ->
 
 aspect_init_after_external_f() ->
     fun(_CommandLine, _Prefix, _Timeout, State) ->
-        case aspect_init() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_init() of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(spiral, MetricPrefix ++ [init], 1, Driver);
             undefined ->
                 ok
@@ -1034,13 +1035,16 @@ aspect_init_after_external_f() ->
 aspect_request_before_internal_f(Ref) ->
     fun(_Type, _Name, _Pattern, _RequestInfo, _Request,
         _Timeout, _Priority, _TransId, Source, State, Dispatcher) ->
-        case aspect_ref_to_object(Ref, Dispatcher) of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_ref_to_object(Ref, Dispatcher) of
             {_, MetricPrefix, Driver} ->
-                case aspect_pid_to_service_id(Source) of
+                case ?MODULE:aspect_pid_to_service_id(Source) of
                     undefined ->
+                        cloudi_service_monitoring:
                         update(spiral, MetricPrefix ++ [request, nonservice],
                                1, Driver);
                     ServiceMetricId ->
+                        cloudi_service_monitoring:
                         update(spiral, MetricPrefix ++ [request |
                                                         ServiceMetricId],
                                1, Driver)
@@ -1054,13 +1058,16 @@ aspect_request_before_internal_f(Ref) ->
 aspect_request_before_external_f() ->
     fun(_Type, _Name, _Pattern, _RequestInfo, _Request,
         _Timeout, _Priority, _TransId, Source, State) ->
-        case aspect_pid_to_object() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_pid_to_object() of
             {_, MetricPrefix, Driver} ->
-                case aspect_pid_to_service_id(Source) of
+                case ?MODULE:aspect_pid_to_service_id(Source) of
                     undefined ->
+                        cloudi_service_monitoring:
                         update(spiral, MetricPrefix ++ [request, nonservice],
                                1, Driver);
                     ServiceMetricId ->
+                        cloudi_service_monitoring:
                         update(spiral, MetricPrefix ++ [request |
                                                         ServiceMetricId],
                                1, Driver)
@@ -1074,8 +1081,10 @@ aspect_request_before_external_f() ->
 aspect_request_after_internal_f(Ref) ->
     fun(_Type, _Name, _Pattern, _RequestInfo, _Request,
         Timeout, _Priority, _TransId, _Source, _Result, State, Dispatcher) ->
-        case aspect_ref_to_object(Ref, Dispatcher) of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_ref_to_object(Ref, Dispatcher) of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(histogram, MetricPrefix ++ [request, timeout],
                        Timeout, Driver);
             undefined ->
@@ -1087,8 +1096,10 @@ aspect_request_after_internal_f(Ref) ->
 aspect_request_after_external_f() ->
     fun(_Type, _Name, _Pattern, _RequestInfo, _Request,
         Timeout, _Priority, _TransId, _Source, _Result, State) ->
-        case aspect_pid_to_object() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_pid_to_object() of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(histogram, MetricPrefix ++ [request, timeout],
                        Timeout, Driver);
             undefined ->
@@ -1099,8 +1110,10 @@ aspect_request_after_external_f() ->
 
 aspect_info_before_internal_f(Ref) ->
     fun(_Request, State, Dispatcher) ->
-        case aspect_ref_to_object(Ref, Dispatcher) of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_ref_to_object(Ref, Dispatcher) of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(spiral, MetricPrefix ++ [info], 1, Driver);
             undefined ->
                 ok
@@ -1110,7 +1123,8 @@ aspect_info_before_internal_f(Ref) ->
 
 aspect_info_after_internal_f(_Ref) ->
     fun(_Request, State, _Dispatcher) ->
-        %case aspect_ref_to_object(Ref, Dispatcher) of
+        % only remote function calls here to allow a module reload
+        %case ?MODULE:aspect_ref_to_object(Ref, Dispatcher) of
         %    {_, MetricPrefix, Driver} ->
         %        ok;
         %    undefined ->
@@ -1121,8 +1135,10 @@ aspect_info_after_internal_f(_Ref) ->
 
 aspect_terminate_before_internal_f() ->
     fun(_Reason, _Timeout, State) ->
-        case aspect_pid_to_object() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_pid_to_object() of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(counter, MetricPrefix ++ [terminate], 1, Driver);
             undefined ->
                 ok
@@ -1132,8 +1148,10 @@ aspect_terminate_before_internal_f() ->
 
 aspect_terminate_before_external_f() ->
     fun(_Reason, _Timeout, State) ->
-        case aspect_pid_to_object() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_pid_to_object() of
             {_, MetricPrefix, Driver} ->
+                cloudi_service_monitoring:
                 update(counter, MetricPrefix ++ [terminate], 1, Driver);
             undefined ->
                 ok
@@ -1145,9 +1163,11 @@ aspect_log_f(OutputTime)
     when OutputTime =:= 'before'; OutputTime =:= 'after' ->
     fun(Level, _Timestamp, _Node, _Pid,
         _Module, _Line, _Function, _Arity, _MetaData, _LogMessage) ->
-        case aspect_cloudi() of
+        % only remote function calls here to allow a module reload
+        case ?MODULE:aspect_cloudi() of
             {_, MetricPrefix, Driver} ->
                 Name = MetricPrefix ++ [logging, output, OutputTime, Level],
+                cloudi_service_monitoring:
                 update(spiral, Name, 1, Driver);
             undefined ->
                 ok
@@ -1251,6 +1271,3 @@ service_metric_id_index(Type, FileName, ID) ->
     end,
     erlang:integer_to_list(Count - 1).
 
-update(Type, Name, Value, Driver) ->
-    cloudi_service_monitoring:update(Type, Name, Value, Driver).
-
