@@ -8,45 +8,31 @@
 %%% (http://tools.ietf.org/html/rfc5849).
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2014-2015, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2015 Michael Truog
-%%% @version 1.5.0 {@date} {@time}
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_oauth1).
@@ -356,7 +342,7 @@ request("verify*", Name, Pattern, RequestHeaders, Request, Timeout,
         #state{url_host = URLHost,
                host = Host} = State, Dispatcher) ->
     [NextName] = cloudi_service_name:parse(Name, Pattern),
-    Method = string:to_upper(cloudi_string:afterr($/, NextName)),
+    Method = cloudi_string:uppercase(cloudi_string:afterr($/, NextName)),
     case url(Method, RequestHeaders, URLHost, Host, Request) of
         {ok, URL} ->
             case oauth_parameters(Method, RequestHeaders, Request) of
@@ -537,12 +523,7 @@ request_initiate_store(Realm, ConsumerKey, SignatureMethod,
     % provide request token and secret
     TokenRequest = token_request(State),
     TokenRequestSecret = token_request_secret(State),
-    {CallbackURL, CallbackQS} = case string:tokens(Callback, "?") of
-        [CallbackURLValue, CallbackQSValue] ->
-            {CallbackURLValue, CallbackQSValue};
-        [_] ->
-            {Callback, ""}
-    end,
+    {CallbackURL, CallbackQS} = cloudi_string:splitl($?, Callback, input),
     case DatabaseModule:token_request_store(Dispatcher, Database,
                                             Realm, ConsumerKey,
                                             SignatureMethod, ClientSharedSecret,
