@@ -123,7 +123,8 @@ init_per_testcase(t_example_without_db_1, Config) ->
             [{mode, example}],
             immediate_closest,
             ?TIMEOUT, ?TIMEOUT, ?TIMEOUT,
-            undefined, undefined, 1, 5, 300, []},
+            undefined, undefined, 1, 5, 300,
+            [{automatic_loading, false}]},
         {internal,
             "/",
             cloudi_service_oauth1,
@@ -146,7 +147,8 @@ init_per_testcase(t_example_without_db_2, Config) ->
             [{mode, example}],
             immediate_closest,
             ?TIMEOUT, ?TIMEOUT, ?TIMEOUT,
-            undefined, undefined, 1, 5, 300, []},
+            undefined, undefined, 1, 5, 300,
+            [{automatic_loading, false}]},
         {internal,
             "/",
             cloudi_service_oauth1,
@@ -169,7 +171,8 @@ init_per_testcase(t_example_with_db_1, Config) ->
             [{mode, example}],
             immediate_closest,
             ?TIMEOUT, ?TIMEOUT, ?TIMEOUT,
-            undefined, undefined, 1, 5, 300, []},
+            undefined, undefined, 1, 5, 300,
+            [{automatic_loading, false}]},
         {internal,
             "/",
             cloudi_service_oauth1,
