@@ -8,45 +8,31 @@
 %%% No disk index is maintained, but an in-memory index is kept.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @copyright 2014-2017 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_write_ahead_logging).
@@ -61,7 +47,7 @@
          store_end/3,
          store_fail/2,
          store_start/2,
-         new/3,
+         new/4,
          update/3]).
 
 % overhead: chunk_size, chunk_size_used
@@ -79,14 +65,14 @@
         retries = 0 :: non_neg_integer()
     }).
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
 -record(state,
     {
-        file = undefined :: undefined | string(),
-        compression = undefined :: undefined | 0..9, % zlib compression level
-        position = undefined :: undefined | non_neg_integer(),
-        chunks = dict:new() :: dict_proxy(cloudi_service:trans_id(), #chunk{}),
-        chunks_free = [] :: list(#chunk{}) % ordered
+        file :: string(),
+        file_size_limit :: pos_integer_64bit(), % bytes
+        compression :: 0..9, % zlib compression level
+        position :: non_neg_integer(),
+        chunks :: #{cloudi_service:trans_id() := #chunk{}},
+        chunks_free :: list(#chunk{}) % ordered
     }).
 
 -type state() :: #state{}.
@@ -116,12 +102,12 @@
 erase(ChunkId,
       #state{file = FilePath,
              chunks = Chunks} = State) ->
-    Chunk = dict:fetch(ChunkId, Chunks),
+    {Chunk, NewChunks} = maps:take(ChunkId, Chunks),
     #chunk{request = ChunkRequest} = Chunk,
     {ok, Fd} = file_open_tmp(FilePath),
     NewState = erase_chunk(Chunk, Fd, State),
     ok = file_close_tmp(FilePath, Fd),
-    {ChunkRequest, NewState#state{chunks = dict:erase(ChunkId, Chunks)}}.
+    {ChunkRequest, NewState#state{chunks = NewChunks}}.
 
 -spec erase_retry(ChunkId :: cloudi_service:trans_id(),
                   RetryMax :: non_neg_integer(),
@@ -132,7 +118,7 @@ erase(ChunkId,
 erase_retry(ChunkId, RetryMax, RetryF,
             #state{file = FilePath,
                    chunks = Chunks} = State) ->
-    Chunk = dict:fetch(ChunkId, Chunks),
+    {Chunk, NewChunks} = maps:take(ChunkId, Chunks),
     #chunk{request = ChunkRequest,
            retries = Retries} = Chunk,
     NewChunkId = case RetryF(ChunkRequest, Retries < RetryMax) of
@@ -146,24 +132,23 @@ erase_retry(ChunkId, RetryMax, RetryF,
             {ok, Fd} = file_open_tmp(FilePath),
             NewState = erase_chunk(Chunk, Fd, State),
             ok = file_close_tmp(FilePath, Fd),
-            NewState#state{chunks = dict:erase(ChunkId, Chunks)};
+            NewState#state{chunks = NewChunks};
         is_binary(NewChunkId) ->
             NewChunk = Chunk#chunk{retries = Retries + 1},
-            NewChunks = dict:erase(ChunkId, Chunks),
-            State#state{chunks = dict:store(NewChunkId, NewChunk, NewChunks)}
+            State#state{chunks = maps:put(NewChunkId, NewChunk, NewChunks)}
     end.
 
 -spec fetch_keys(State :: #state{}) ->
     list(cloudi_service:trans_id()).
 
 fetch_keys(#state{chunks = Chunks}) ->
-    lists:sort(dict:fetch_keys(Chunks)). % oldest -> newest
+    lists:sort(maps:keys(Chunks)). % oldest -> newest
 
 -spec size(State :: #state{}) ->
     non_neg_integer().
 
 size(#state{chunks = Chunks}) ->
-    dict:size(Chunks).
+    maps:size(Chunks).
 
 -spec size_free(State :: #state{}) ->
     non_neg_integer().
@@ -178,7 +163,7 @@ size_free(#state{chunks_free = ChunksFree}) ->
 
 store_end(ChunkId, Chunk,
           #state{chunks = Chunks} = State) ->
-    State#state{chunks = dict:store(ChunkId, Chunk, Chunks)}.
+    State#state{chunks = maps:put(ChunkId, Chunk, Chunks)}.
 
 -spec store_fail(Chunk :: #chunk{},
                  State :: #state{}) ->
@@ -192,10 +177,11 @@ store_fail(Chunk, #state{file = FilePath} = State) ->
 
 -spec store_start(ChunkRequest :: cloudi_service_queue:request(),
                   State :: #state{}) ->
-    {#chunk{}, #state{}}.
+    {#chunk{}, #state{}} | full.
 
 store_start(ChunkRequest,
             #state{file = FilePath,
+                   file_size_limit = FileSizeLimit,
                    compression = Compression,
                    position = Position,
                    chunks_free = ChunksFree} = State) ->
@@ -204,6 +190,9 @@ store_start(ChunkRequest,
                                       [{compressed, Compression}]),
     ChunkSizeUsed = erlang:byte_size(ChunkData),
     case chunk_free_check(ChunksFree, ChunkSizeUsed) of
+        false
+            when Position + (?CHUNK_OVERHEAD + ChunkSizeUsed) > FileSizeLimit ->
+            full;
         false ->
             ChunkSize = ChunkSizeUsed,
             NewPosition = chunk_write(ChunkSize, ChunkSizeUsed,
@@ -223,27 +212,28 @@ store_start(ChunkRequest,
     end.
 
 -spec new(FilePath :: string(),
+          FileSizeLimit :: 1024..?MAX_64BITS,
           Compression :: 0..9,
           RetryF :: retry_function()) ->
     #state{}.
 
-new(FilePath, Compression, RetryF)
-    when is_integer(Compression), Compression >= 0, Compression =< 9,
+new(FilePath, FileSizeLimit, Compression, RetryF)
+    when is_integer(FileSizeLimit),
+         FileSizeLimit >= 1024, FileSizeLimit =< ?MAX_64BITS,
+         is_integer(Compression), Compression >= 0, Compression =< 9,
          is_function(RetryF, 2) ->
-    State = #state{},
-    #state{chunks = Chunks,
-           chunks_free = ChunksFree} = State,
     {ok, Fd} = file_open_copy(FilePath),
     {ok,
      Position,
-     NewChunks,
-     NewChunksFree} = chunks_recover(Chunks, ChunksFree, Fd, RetryF),
+     Chunks,
+     ChunksFree} = chunks_recover(#{}, [], Fd, RetryF),
     ok = file_close_tmp(FilePath, Fd),
-    State#state{file = FilePath,
-                compression = Compression,
-                position = Position,
-                chunks = NewChunks,
-                chunks_free = NewChunksFree}.
+    #state{file = FilePath,
+           file_size_limit = FileSizeLimit,
+           compression = Compression,
+           position = Position,
+           chunks = Chunks,
+           chunks_free = ChunksFree}.
 
 -spec update(ChunkId :: cloudi_service:trans_id(),
              UpdateF :: update_function(),
@@ -253,10 +243,8 @@ new(FilePath, Compression, RetryF)
 update(ChunkId, UpdateF,
        #state{file = FilePath,
               compression = Compression,
-              position = Position,
-              chunks = Chunks,
-              chunks_free = ChunksFree} = State) ->
-    Chunk = dict:fetch(ChunkId, Chunks),
+              chunks = Chunks} = State) ->
+    Chunk = maps:get(ChunkId, Chunks),
     #chunk{request = ChunkRequest} = Chunk,
     case UpdateF(ChunkRequest) of
         undefined ->
@@ -264,32 +252,38 @@ update(ChunkId, UpdateF,
             {undefined, NewState};
         {NewChunkId, NewChunkRequest} ->
             {ok, Fd} = file_open_tmp(FilePath),
+            % erase previous entry
+            NextState = erase_chunk(Chunk, Fd, State),
+            #state{position = Position,
+                   chunks_free = ChunksFree} = NextState,
             % store update
             NewChunkData = erlang:term_to_binary(NewChunkRequest,
                                                  [{compressed, Compression}]),
             NewChunkSizeUsed = erlang:byte_size(NewChunkData),
-            NextState = case chunk_free_check(ChunksFree, NewChunkSizeUsed) of
+            NewState = case chunk_free_check(ChunksFree, NewChunkSizeUsed) of
                 false ->
+                    % not checking NewChunkSizeUsed with file_size_limit
+                    % due to the operation being an update
+                    % (so it is possible to have the file size exceed
+                    %  file_size_limit here)
                     NewChunkSize = NewChunkSizeUsed,
                     NewPosition = chunk_write(NewChunkSize, NewChunkSizeUsed,
                                               NewChunkData, Position, Fd),
                     NewChunk = #chunk{size = NewChunkSize,
                                       position = Position,
                                       request = NewChunkRequest},
-                    NewChunks = dict:store(NewChunkId, NewChunk, Chunks),
-                    State#state{chunks = NewChunks,
-                                position = NewPosition};
+                    NewChunks = maps:put(NewChunkId, NewChunk, Chunks),
+                    NextState#state{chunks = NewChunks,
+                                    position = NewPosition};
                 {#chunk{size = ChunkSize,
                         position = ChunkPosition} = ChunkFree, NewChunksFree} ->
                     chunk_write(ChunkSize, NewChunkSizeUsed,
                                 NewChunkData, ChunkPosition, Fd),
                     NewChunk = ChunkFree#chunk{request = NewChunkRequest},
-                    NewChunks = dict:store(NewChunkId, NewChunk, Chunks),
-                    State#state{chunks = NewChunks,
-                                chunks_free = NewChunksFree}
+                    NewChunks = maps:put(NewChunkId, NewChunk, Chunks),
+                    NextState#state{chunks = NewChunks,
+                                    chunks_free = NewChunksFree}
             end,
-            % erase previous entry
-            NewState = erase_chunk(Chunk, Fd, NextState),
             ok = file_close_tmp(FilePath, Fd),
             {NewChunkRequest, NewState}
     end.
@@ -398,7 +392,7 @@ chunk_recover_used(Position, ChunkSize, ChunkSizeUsed,
                                    position = Position,
                                    request = ChunkRequest},
                     chunks_recover(NewPosition,
-                                   dict:store(ChunkId, Chunk, Chunks),
+                                   maps:put(ChunkId, Chunk, Chunks),
                                    ChunksFree, Fd, RetryF);
                 {ok, ChunkId} ->
                     ChunkEnd = (ChunkSize - ChunkSizeUsed),
@@ -408,7 +402,7 @@ chunk_recover_used(Position, ChunkSize, ChunkSizeUsed,
                                            position = Position,
                                            request = ChunkRequest},
                             chunks_recover(NewPosition,
-                                           dict:store(ChunkId, Chunk, Chunks),
+                                           maps:put(ChunkId, Chunk, Chunks),
                                            ChunksFree, Fd, RetryF);
                         {error, Reason} ->
                             {error, {chunk_corrupt, Reason}}
