@@ -14,45 +14,31 @@
 %%% this service will timeout (i.e., receive no response).
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2013-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @copyright 2013-2017 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_quorum).
@@ -130,7 +116,6 @@
         dest :: pid()
     }).
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
 -record(state,
     {
         quorum :: byzantine | number(),
@@ -142,18 +127,18 @@
         failures_source_die :: boolean(),
         failures_source_max_count :: pos_integer(),
         failures_source_max_period :: infinity | pos_integer(),
-        failures_source = dict:new()
-            :: dict_proxy(pid(), list(erlang:timestamp())),
+        failures_source = #{}
+            :: #{pid() := list(cloudi_timestamp:seconds_monotonic())},
         failures_dest_die :: boolean(),
         failures_dest_max_count :: pos_integer(),
         failures_dest_max_period :: infinity | pos_integer(),
-        failures_dest = dict:new()
-            :: dict_proxy(pid(), list(erlang:timestamp())),
-        requests = dict:new()
-            :: dict_proxy(cloudi_service:trans_id(), #request{}), % orig
-        pending = dict:new()
-            :: dict_proxy(cloudi_service:trans_id(), % new
-                          cloudi_service:trans_id()) % orig
+        failures_dest = #{}
+            :: #{pid() := list(cloudi_timestamp:seconds_monotonic())},
+        requests = #{}
+            :: #{cloudi_service:trans_id() := #request{}}, % orig
+        pending = #{}
+            :: #{cloudi_service:trans_id() := % new
+                 cloudi_service:trans_id()}   % orig
     }).
 
 %%%------------------------------------------------------------------------
@@ -298,9 +283,9 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                   requests = Requests,
                                   pending = Pending} = State,
                            Dispatcher) ->
-    #pending{trans_id = TransId,
-             dest = DstPid} = dict:fetch(QuorumTransId, Pending),
-    NewPending = dict:erase(QuorumTransId, Pending),
+    {#pending{trans_id = TransId,
+              dest = DstPid},
+     NewPending} = maps:take(QuorumTransId, Pending),
     #request{% return data
              type = Type,
              name = Name,
@@ -312,7 +297,7 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
              count_responses = CountResponses,
              count_correct = CountCorrect,
              responses = Responses,
-             returned = Returned} = Request = dict:fetch(TransId, Requests),
+             returned = Returned} = Request = maps:get(TransId, Requests),
     NewCountResponses = CountResponses + 1,
     case validate(ResponseInfoF, ResponseF,
                   ResponseInfo, Response) of
@@ -355,24 +340,24 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                     end,
                     if
                         NewCountResponses == CountTotal ->
-                            dict:erase(TransId, Requests);
+                            maps:remove(TransId, Requests);
                         NewCountResponses < CountTotal ->
                             NewResponses = orddict:store(Key, Count, Responses),
-                            dict:store(TransId,
-                                       Request#request{
-                                           count_responses = NewCountResponses,
-                                           count_correct = NewCountCorrect,
-                                           responses = NewResponses,
-                                           returned = NewReturned},
-                                       Requests)
+                            maps:put(TransId,
+                                     Request#request{
+                                         count_responses = NewCountResponses,
+                                         count_correct = NewCountCorrect,
+                                         responses = NewResponses,
+                                         returned = NewReturned},
+                                     Requests)
                     end;
                 NewCountResponses == CountTotal ->
-                    dict:erase(TransId, Requests);
+                    maps:remove(TransId, Requests);
                 NewCountResponses < CountTotal ->
-                    dict:store(TransId,
-                               Request#request{
-                                   count_responses = NewCountResponses},
-                               Requests)
+                    maps:put(TransId,
+                             Request#request{
+                                 count_responses = NewCountResponses},
+                             Requests)
             end,
             {noreply, State#state{requests = NewRequests,
                                   pending = NewPending}};
@@ -402,13 +387,13 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
             end,
             NewRequests = if
                 NewCountResponses == CountTotal ->
-                    dict:erase(TransId, Requests);
+                    maps:remove(TransId, Requests);
                 NewCountResponses < CountTotal ->
-                    dict:store(TransId,
-                               Request#request{
-                                   count_responses = NewCountResponses,
-                                   returned = NewReturned},
-                               Requests)
+                    maps:put(TransId,
+                             Request#request{
+                                 count_responses = NewCountResponses,
+                                 returned = NewReturned},
+                             Requests)
             end,
             {noreply, State#state{failures_source = NewFailuresSrc,
                                   failures_dest = NewFailuresDst,
@@ -433,9 +418,9 @@ cloudi_service_handle_info(#timeout_async_active{trans_id = QuorumTransId},
                                   requests = Requests,
                                   pending = Pending} = State,
                            Dispatcher) ->
-    #pending{trans_id = TransId,
-             dest = DstPid} = dict:fetch(QuorumTransId, Pending),
-    NewPending = dict:erase(QuorumTransId, Pending),
+    {#pending{trans_id = TransId,
+              dest = DstPid},
+     NewPending} = maps:take(QuorumTransId, Pending),
     #request{% return data
              type = Type,
              name = Name,
@@ -447,7 +432,7 @@ cloudi_service_handle_info(#timeout_async_active{trans_id = QuorumTransId},
              count_total = CountTotal,
              count_responses = CountResponses,
              count_correct = CountCorrect,
-             returned = Returned} = Request = dict:fetch(TransId, Requests),
+             returned = Returned} = Request = maps:get(TransId, Requests),
     NewCountResponses = CountResponses + 1,
     {DeadSrc, NewFailuresSrc} = failure(FailuresSrcDie,
                                         FailuresSrcMaxCount,
@@ -474,12 +459,12 @@ cloudi_service_handle_info(#timeout_async_active{trans_id = QuorumTransId},
     end,
     NewRequests = if
         NewCountResponses == CountTotal ->
-            dict:erase(TransId, Requests);
+            maps:remove(TransId, Requests);
         NewCountResponses < CountTotal ->
-            dict:store(TransId,
-                       Request#request{count_responses = NewCountResponses,
-                                       returned = NewReturned},
-                       Requests)
+            maps:put(TransId,
+                     Request#request{count_responses = NewCountResponses,
+                                     returned = NewReturned},
+                     Requests)
     end,
     {noreply, State#state{failures_source = NewFailuresSrc,
                           failures_dest = NewFailuresDst,
@@ -494,13 +479,13 @@ cloudi_service_handle_info({'DOWN', _MonitorRef, process, Pid, _Info},
                            _Dispatcher) ->
     NewFailuresSrc = if
         FailuresSrcDie =:= true ->
-            dict:erase(Pid, FailuresSrc);
+            maps:remove(Pid, FailuresSrc);
         FailuresSrcDie =:= false ->
             FailuresSrc
     end,
     NewFailuresDst = if
         FailuresDstDie =:= true ->
-            dict:erase(Pid, FailuresDst);
+            maps:remove(Pid, FailuresDst);
         FailuresDstDie =:= false ->
             FailuresDst
     end,
@@ -533,10 +518,10 @@ mcast_send([{_, DstPid} = PatternPid | PatternPids],
                                           Timeout, Priority, PatternPid) of
         {ok, QuorumTransId} ->
             mcast_send(PatternPids, CountSent + 1,
-                       dict:store(QuorumTransId,
-                                  #pending{trans_id = TransId,
-                                           dest = DstPid},
-                                  Pending),
+                       maps:put(QuorumTransId,
+                                #pending{trans_id = TransId,
+                                         dest = DstPid},
+                                Pending),
                        QuorumName,
                        RequestInfo, Request,
                        Timeout, Priority, TransId, Dispatcher);
@@ -558,29 +543,29 @@ mcast(PatternPids, QuorumName,
                               Timeout, Priority, TransId, Dispatcher),
     if
         CountSent >= CountRequired ->
-            NewRequests = dict:store(TransId,
-                                     #request{type = Type,
-                                              name = Name,
-                                              pattern = Pattern,
-                                              timeout = Timeout,
-                                              source = SrcPid,
-                                              count_required = CountRequired,
-                                              count_total = CountSent,
-                                              returned = false},
-                                     Requests),
+            NewRequests = maps:put(TransId,
+                                   #request{type = Type,
+                                            name = Name,
+                                            pattern = Pattern,
+                                            timeout = Timeout,
+                                            source = SrcPid,
+                                            count_required = CountRequired,
+                                            count_total = CountSent,
+                                            returned = false},
+                                   Requests),
             {noreply, State#state{requests = NewRequests,
                                   pending = NewPending}};
         true ->
-            NewRequests = dict:store(TransId,
-                                     #request{type = Type,
-                                              name = Name,
-                                              pattern = Pattern,
-                                              timeout = Timeout,
-                                              source = SrcPid,
-                                              count_required = CountRequired,
-                                              count_total = CountSent,
-                                              returned = true},
-                                     Requests),
+            NewRequests = maps:put(TransId,
+                                   #request{type = Type,
+                                            name = Name,
+                                            pattern = Pattern,
+                                            timeout = Timeout,
+                                            source = SrcPid,
+                                            count_required = CountRequired,
+                                            count_total = CountSent,
+                                            returned = true},
+                                   Requests),
             {reply, <<>>, State#state{requests = NewRequests,
                                       pending = NewPending}}
     end.
@@ -620,8 +605,8 @@ failure(false, _, _, _, Failures) ->
 failure(true, MaxCount, MaxPeriod, Pid, Failures) ->
     case erlang:is_process_alive(Pid) of
         true ->
-            SecondsNow = cloudi_timestamp:seconds(),
-            case dict:find(Pid, Failures) of
+            SecondsNow = cloudi_timestamp:seconds_monotonic(),
+            case maps:find(Pid, Failures) of
                 {ok, FailureList} ->
                     failure_check(SecondsNow, FailureList,
                                   MaxCount, MaxPeriod, Pid, Failures);
@@ -635,7 +620,7 @@ failure(true, MaxCount, MaxPeriod, Pid, Failures) ->
     end.
 
 failure_store(FailureList, FailureCount, MaxCount, Pid, Failures) ->
-    NewFailures = dict:store(Pid, FailureList, Failures),
+    NewFailures = maps:put(Pid, FailureList, Failures),
     if
         FailureCount == MaxCount ->
             failure_kill(Pid),
@@ -650,8 +635,9 @@ failure_check(SecondsNow, FailureList, MaxCount, infinity, Pid, Failures) ->
                   MaxCount, Pid, Failures);
 failure_check(SecondsNow, FailureList, MaxCount, MaxPeriod, Pid, Failures) ->
     {NewFailureCount,
-     NewFailureList} = cloudi_timestamp:seconds_filter(FailureList,
-                                                       SecondsNow, MaxPeriod),
+     NewFailureList} = cloudi_timestamp:seconds_filter_monotonic(FailureList,
+                                                                 SecondsNow,
+                                                                 MaxPeriod),
     failure_store([SecondsNow | NewFailureList], NewFailureCount + 1,
                   MaxCount, Pid, Failures).
 
