@@ -6,45 +6,31 @@
 %%% ==Basic CloudI TCP Integration==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2013-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2013-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_tcp).
@@ -125,8 +111,6 @@
 -define(DEFAULT_DEBUG,                       false). % log output for debugging
 -define(DEFAULT_DEBUG_LEVEL,                 trace).
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
-
 -record(state,
     {
         scope = undefined :: undefined | atom(),
@@ -149,7 +133,7 @@
             :: undefined | cloudi_service:service_name(),
         connection_count = 0 :: non_neg_integer(),
         connection_max = undefined :: undefined | pos_integer(),
-        requests = dict:new(),
+        requests = #{},
         destination_set = undefined :: undefined | boolean(),
         protocol_id_check = undefined
             :: undefined |
@@ -181,11 +165,10 @@
             :: undefined | cloudi:message_service_request(),
         response_lookup
             :: undefined |
-               dict_proxy(any(),
-                          {cloudi:message_service_request(), reference()}),
+               #{any() := {cloudi:message_service_request(), reference()}},
         recv_timeouts
             :: undefined |
-               dict_proxy(cloudi:trans_id(), reference()),
+               #{cloudi:trans_id() := reference()},
         queued
             :: undefined |
                cloudi_x_pqueue4:cloudi_x_pqueue4(
@@ -498,14 +481,14 @@ socket_loop_init_set(true, Scope,
             ResponseLookup = if
                 Subscribed =:= true,
                 ProtocolIdCheck /= undefined ->
-                    dict:new();
+                    #{};
                 true ->
                     undefined
             end,
             RecvTimeouts = if
                 Subscribed =:= true,
                 ProtocolIdCheck =:= undefined ->
-                    dict:new();
+                    #{};
                 true ->
                     undefined
             end,
@@ -607,7 +590,7 @@ socket_loop_out_request({Type, Name, Pattern, RequestInfo, RequestProtocol,
     socket_debug_log(DebugLevel,
                     "socket out request ~p", [Request]),
     State#state_socket{
-        response_lookup = dict:store(ID, {T, ResponseTimer}, ResponseLookup)};
+        response_lookup = maps:put(ID, {T, ResponseTimer}, ResponseLookup)};
 socket_loop_out_request({_, _, _, _, Request,
                          Timeout, Priority, TransId, _} = T,
                         #state_socket{
@@ -617,7 +600,7 @@ socket_loop_out_request({_, _, _, _, Request,
                             queued = Queue} = State)
     when is_binary(Request), Timeout > 0 ->
     State#state_socket{
-        recv_timeouts = dict:store(TransId,
+        recv_timeouts = maps:put(TransId,
             erlang:send_after(Timeout, self(),
                 {'cloudi_service_recv_timeout', Priority, TransId}),
             RecvTimeouts),
@@ -719,7 +702,7 @@ socket_loop_in_data(Data,
         {incoming, Request} ->
             {undefined, undefined, Request};
         {ID, Response} ->
-            case dict:find(ID, ResponseLookup) of
+            case maps:find(ID, ResponseLookup) of
                 {ok, ResponseData} ->
                     {ID, ResponseData, Response};
                 error ->
@@ -742,7 +725,7 @@ socket_loop_in_data(Data,
             ok = socket_loop_in_response(T, ResponseTimer,
                                          Info, Value, DebugLevel),
             State#state_socket{
-                response_lookup = dict:erase(LookupID, ResponseLookup)}
+                response_lookup = maps:remove(LookupID, ResponseLookup)}
     end.
 
 socket_loop(#state_socket{
@@ -767,7 +750,7 @@ socket_loop(#state_socket{
             F = fun({_, {_, _, _, _, _, _, _, Id, _}}) -> Id == TransId end,
             {_, NewQueue} = cloudi_x_pqueue4:remove_unique(F, Priority, Queue),
             socket_loop(State#state_socket{
-                            recv_timeouts = dict:erase(TransId, RecvTimeouts),
+                            recv_timeouts = maps:remove(TransId, RecvTimeouts),
                             queued = NewQueue});
         response_timeout ->
             true = ResponsePending,
@@ -779,7 +762,7 @@ socket_loop(#state_socket{
         {response_timeout, ID} ->
             false = ResponsePending,
             socket_loop(State#state_socket{
-                            response_lookup = dict:erase(ID, ResponseLookup)})
+                            response_lookup = maps:remove(ID, ResponseLookup)})
     after
         TimeoutRecv ->
             socket_loop_terminate(normal, State)
@@ -855,14 +838,14 @@ socket_process_queue(#state_socket{recv_timeouts = RecvTimeouts,
             State#state_socket{queued = NewQueue};
         {{value, {Type, Name, Pattern, RequestInfo, Request,
                   _, Priority, TransId, Pid}}, NewQueue} ->
-            Timeout = case erlang:cancel_timer(dict:fetch(TransId,
-                                                          RecvTimeouts)) of
+            Timeout = case erlang:cancel_timer(maps:get(TransId,
+                                                        RecvTimeouts)) of
                 false ->
                     0;
                 V ->
                     V
             end,
-            NewRecvTimeouts = dict:erase(TransId, RecvTimeouts),
+            NewRecvTimeouts = maps:remove(TransId, RecvTimeouts),
             socket_loop_out_request({Type, Name, Pattern, RequestInfo, Request,
                                      Timeout, Priority, TransId, Pid},
                                     State#state_socket{
