@@ -6,45 +6,31 @@
 %%% ==CloudI Validate Service==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @copyright 2015-2017 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_validate).
@@ -103,7 +89,6 @@
         destination :: pid()
     }).
 
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
 -record(state,
     {
         validate_request_info :: undefined | fun((any()) -> boolean()),
@@ -113,15 +98,15 @@
         failures_source_die :: boolean(),
         failures_source_max_count :: pos_integer(),
         failures_source_max_period :: infinity | pos_integer(),
-        failures_source = dict:new()
-            :: dict_proxy(pid(), list(erlang:timestamp())),
+        failures_source = #{}
+            :: #{pid() := list(cloudi_timestamp:seconds_monotonic())},
         failures_dest_die :: boolean(),
         failures_dest_max_count :: pos_integer(),
         failures_dest_max_period :: infinity | pos_integer(),
-        failures_dest = dict:new()
-            :: dict_proxy(pid(), list(erlang:timestamp())),
-        requests = dict:new()
-            :: dict_proxy(cloudi_service:trans_id(), #request{})
+        failures_dest = #{}
+            :: #{pid() := list(cloudi_timestamp:seconds_monotonic())},
+        requests = #{}
+            :: #{cloudi_service:trans_id() := #request{}}
     }).
 
 %%%------------------------------------------------------------------------
@@ -206,9 +191,9 @@ cloudi_service_handle_request(Type, Name, Pattern, RequestInfo, Request,
                                                        source = SrcPid,
                                                        destination = DstPid},
                             {noreply,
-                             State#state{requests = dict:store(ValidateTransId,
-                                                               ValidateRequest,
-                                                               Requests)}};
+                             State#state{requests = maps:put(ValidateTransId,
+                                                             ValidateRequest,
+                                                             Requests)}};
                         {error, timeout} ->
                             request_failed(SrcPid, State)
                     end;
@@ -239,13 +224,13 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                   failures_dest = FailuresDst,
                                   requests = Requests} = State,
                            Dispatcher) ->
-    #request{type = Type,
-             name = Name,
-             pattern = Pattern,
-             trans_id = TransId,
-             source = SrcPid,
-             destination = DstPid} = dict:fetch(ValidateTransId, Requests),
-    NewRequests = dict:erase(ValidateTransId, Requests),
+    {#request{type = Type,
+              name = Name,
+              pattern = Pattern,
+              trans_id = TransId,
+              source = SrcPid,
+              destination = DstPid},
+     NewRequests} = maps:take(ValidateTransId, Requests),
     case validate(ResponseInfoF, ResponseF,
                   ResponseInfo, Response) of
         true ->
@@ -291,14 +276,14 @@ cloudi_service_handle_info(#timeout_async_active{trans_id = ValidateTransId},
                                   failures_dest = FailuresDst,
                                   requests = Requests} = State,
                            Dispatcher) ->
-    #request{type = Type,
-             name = Name,
-             pattern = Pattern,
-             timeout = Timeout,
-             trans_id = TransId,
-             source = SrcPid,
-             destination = DstPid} = dict:fetch(ValidateTransId, Requests),
-    NewRequests = dict:erase(ValidateTransId, Requests),
+    {#request{type = Type,
+              name = Name,
+              pattern = Pattern,
+              timeout = Timeout,
+              trans_id = TransId,
+              source = SrcPid,
+              destination = DstPid},
+     NewRequests} = maps:take(ValidateTransId, Requests),
     {DeadSrc, NewFailuresSrc} = failure(FailuresSrcDie,
                                         FailuresSrcMaxCount,
                                         FailuresSrcMaxPeriod,
@@ -327,13 +312,13 @@ cloudi_service_handle_info({'DOWN', _MonitorRef, process, Pid, _Info},
                            _Dispatcher) ->
     NewFailuresSrc = if
         FailuresSrcDie =:= true ->
-            dict:erase(Pid, FailuresSrc);
+            maps:remove(Pid, FailuresSrc);
         FailuresSrcDie =:= false ->
             FailuresSrc
     end,
     NewFailuresDst = if
         FailuresDstDie =:= true ->
-            dict:erase(Pid, FailuresDst);
+            maps:remove(Pid, FailuresDst);
         FailuresDstDie =:= false ->
             FailuresDst
     end,
@@ -386,8 +371,8 @@ failure(false, _, _, _, Failures) ->
 failure(true, MaxCount, MaxPeriod, Pid, Failures) ->
     case erlang:is_process_alive(Pid) of
         true ->
-            SecondsNow = cloudi_timestamp:seconds(),
-            case dict:find(Pid, Failures) of
+            SecondsNow = cloudi_timestamp:seconds_monotonic(),
+            case maps:find(Pid, Failures) of
                 {ok, FailureList} ->
                     failure_check(SecondsNow, FailureList,
                                   MaxCount, MaxPeriod, Pid, Failures);
@@ -401,7 +386,7 @@ failure(true, MaxCount, MaxPeriod, Pid, Failures) ->
     end.
 
 failure_store(FailureList, FailureCount, MaxCount, Pid, Failures) ->
-    NewFailures = dict:store(Pid, FailureList, Failures),
+    NewFailures = maps:put(Pid, FailureList, Failures),
     if
         FailureCount == MaxCount ->
             failure_kill(Pid),
@@ -416,8 +401,9 @@ failure_check(SecondsNow, FailureList, MaxCount, infinity, Pid, Failures) ->
                   MaxCount, Pid, Failures);
 failure_check(SecondsNow, FailureList, MaxCount, MaxPeriod, Pid, Failures) ->
     {NewFailureCount,
-     NewFailureList} = cloudi_timestamp:seconds_filter(FailureList,
-                                                       SecondsNow, MaxPeriod),
+     NewFailureList} = cloudi_timestamp:seconds_filter_monotonic(FailureList,
+                                                                 SecondsNow,
+                                                                 MaxPeriod),
     failure_store([SecondsNow | NewFailureList], NewFailureCount + 1,
                   MaxCount, Pid, Failures).
 
