@@ -7,45 +7,31 @@
 %%% Provide a way of sending/receiving through ZeroMQ.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2014, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2014 Michael Truog
-%%% @version 1.4.0 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_zeromq).
@@ -70,7 +56,6 @@
                                                     % meta-data through ZeroMQ
                                                     % w/4-byte big endian header
 -type socket() :: {pos_integer(), any()}.
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
 -record(state,
     {
         context :: erlzmq:erlzmq_context(),
@@ -82,14 +67,13 @@
         request :: cloudi_x_trie:cloudi_x_trie(),
         % Name -> Socket
         push :: cloudi_x_trie:cloudi_x_trie(),
-        receives :: dict_proxy(socket(),
-                               {reply, string()} |
-                               {subscribe,
-                                {non_neg_integer(), tuple(),
-                                 dict_proxy(binary(), string())}} |
-                               {request, fun((binary(), binary()) -> ok)}),
-        reply_replies = dict:new() :: dict_proxy(cloudi_service:trans_id(),
-                                                 socket())
+        receives :: #{socket() :=
+                      {reply, string()} |
+                      {subscribe,
+                       {non_neg_integer(), tuple(),
+                        #{binary() := string()}}} |
+                      {request, fun((binary(), binary()) -> ok)}},
+        reply_replies = #{} :: #{cloudi_service:trans_id() := socket()}
     }).
 
 %%%------------------------------------------------------------------------
@@ -123,7 +107,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
 
     {ok, Context} = erlzmq:context(),
     Service = cloudi_service:self(Dispatcher),
-    ReceivesZMQ1 = dict:new(),
+    ReceivesZMQ1 = #{},
     Publish = lists:foldl(fun({publish,
                                {[{[I1a | _], [I1b | _]} | _] = NamePairL,
                                 [[I2 | _] | _] = EndpointL}}, D) ->
@@ -149,13 +133,13 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
         NameLookup = lists:foldl(fun({NameInternal, NameExternal}, DD) ->
             NameExternalBin = erlang:list_to_binary(NameExternal),
             ok = erlzmq:setsockopt(S, subscribe, NameExternalBin),
-            dict:append(NameExternalBin, Prefix ++ NameInternal, DD)
-        end, dict:new(), NamePairL),
-        NameL = dict:fetch_keys(NameLookup),
+            maps_append(NameExternalBin, Prefix ++ NameInternal, DD)
+        end, #{}, NamePairL),
+        NameL = maps:keys(NameLookup),
         NameMax = lists:foldl(fun(N, M) ->
             erlang:max(erlang:byte_size(N), M)
         end, 0, NameL),
-        dict:store(S, {subscribe, {NameMax,
+        maps:put(S, {subscribe, {NameMax,
                                    binary:compile_pattern(NameL),
                                    NameLookup}}, D)
     end, ReceivesZMQ1, SubscribeL),
@@ -178,7 +162,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
         lists:foreach(fun(Endpoint) ->
             ok = erlzmq:connect(S, Endpoint)
         end, EndpointL),
-        dict:store(S, {reply, Prefix ++ Name}, D)
+        maps:put(S, {reply, Prefix ++ Name}, D)
     end, ReceivesZMQ2, ReplyL),
     Push = lists:foldl(fun({push,
                             {[I1 | _] = Name,
@@ -199,7 +183,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
         lists:foreach(fun(Endpoint) ->
             ok = erlzmq:connect(S, Endpoint)
         end, EndpointL),
-        dict:store(S, {pull, Prefix ++ Name}, D)
+        maps:put(S, {pull, Prefix ++ Name}, D)
     end, ReceivesZMQ3, PullL),
     {ok, #state{context = Context,
                 endian = Endian,
@@ -242,9 +226,9 @@ cloudi_service_handle_request(Type, Name, Pattern, RequestInfo, Request,
                                               ResponseInfo, Response,
                                               Timeout, TransId, Pid)
             end,
-            {noreply, State#state{receives = dict:store(RequestS,
-                                                        {request, F},
-                                                        ReceivesZMQ)}};
+            {noreply, State#state{receives = maps:put(RequestS,
+                                                      {request, F},
+                                                      ReceivesZMQ)}};
         error ->
             % successful publish operations don't need to store a response
             % erroneous synchronous operations will get a timeout
@@ -257,12 +241,12 @@ cloudi_service_handle_info({zmq, S, Incoming, _},
                                   receives = ReceivesZMQ,
                                   reply_replies = ReplyReplies} = State,
                            Dispatcher) ->
-    case dict:find(S, ReceivesZMQ) of
+    case maps:find(S, ReceivesZMQ) of
         {ok, {request, F}} ->
             {ResponseInfo, Response} = incoming(ProcessMetaData, Endian,
                                                 Incoming),
             F(ResponseInfo, Response),
-            {noreply, State#state{receives = dict:erase(S, ReceivesZMQ)}};
+            {noreply, State#state{receives = maps:remove(S, ReceivesZMQ)}};
         {ok, {reply, Name}} ->
             {RequestInfo, Request} = incoming(ProcessMetaData, Endian,
                                               Incoming),
@@ -271,8 +255,8 @@ cloudi_service_handle_info({zmq, S, Incoming, _},
                                                   undefined, undefined) of
                 {ok, TransId} ->
                     {noreply,
-                     State#state{reply_replies = dict:store(TransId, S,
-                                                            ReplyReplies)}};
+                     State#state{reply_replies = maps:put(TransId, S,
+                                                          ReplyReplies)}};
                 {error, _} ->
                     ok = erlzmq:send(S, <<>>),
                     {noreply, State}
@@ -294,7 +278,7 @@ cloudi_service_handle_info({zmq, S, Incoming, _},
                 cloudi_service:send_async(Dispatcher, Name,
                                           RequestInfo, Request,
                                           undefined, undefined)
-            end, dict:fetch(NameZMQ, Lookup)),
+            end, maps:get(NameZMQ, Lookup)),
             {noreply, State}
     end;
 
@@ -307,16 +291,16 @@ cloudi_service_handle_info({'return_async_active', _Name, _Pattern,
                            _Dispatcher) ->
     true = is_binary(Response),
     Outgoing = outgoing(ProcessMetaData, Endian, ResponseInfo, Response),
-    S = dict:fetch(TransId, ReplyReplies),
+    {S, NewReplyReplies} = maps:take(TransId, ReplyReplies),
     ok = erlzmq:send(S, Outgoing),
-    {noreply, State#state{reply_replies = dict:erase(TransId, ReplyReplies)}};
+    {noreply, State#state{reply_replies = NewReplyReplies}};
 
 cloudi_service_handle_info({'timeout_async_active', TransId},
                            #state{reply_replies = ReplyReplies} = State,
                            _Dispatcher) ->
-    S = dict:fetch(TransId, ReplyReplies),
+    {S, NewReplyReplies} = maps:take(TransId, ReplyReplies),
     ok = erlzmq:send(S, <<>>),
-    {noreply, State#state{reply_replies = dict:erase(TransId, ReplyReplies)}};
+    {noreply, State#state{reply_replies = NewReplyReplies}};
 
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
     ?LOG_WARN("Unknown info \"~p\"", [Request]),
@@ -342,7 +326,7 @@ cloudi_service_terminate(_Reason, _Timeout,
     cloudi_x_trie:foreach(fun(_, S) ->
         erlzmq:close(S)
     end, Push),
-    dict:map(fun(S, _) ->
+    maps:map(fun(S, _) ->
         erlzmq:close(S)
     end, ReceivesZMQ),
     erlzmq:term(Context),
@@ -398,3 +382,6 @@ incoming_metadata_split(MetaDataSize, IncomingRest) ->
     Request = erlang:binary_part(IncomingRest, {Size, MetaDataSize - Size}),
     {RequestInfo, Request}.
 
+maps_append(K, V, M) ->
+    maps:update_with(K, fun (L) -> L ++ [V] end, M).
+
