@@ -17,45 +17,31 @@
 %%% methods are provided as specified within the RFC.
 %%% @end
 %%%
-%%% BSD LICENSE
+%%% MIT License
 %%%
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
 %%%
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
 %%%
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
 %%%
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.7.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_uuid).
@@ -94,19 +80,24 @@
          test/0]).
 
 -ifdef(ERLANG_OTP_VERSION_16).
--define(TIMESTAMP_ERLANG_NOW, true).
 -else.
 -ifdef(ERLANG_OTP_VERSION_17).
--define(TIMESTAMP_ERLANG_NOW, true).
 -else.
 -define(ERLANG_OTP_VERSION_18_FEATURES, true).
+-ifdef(ERLANG_OTP_VERSION_18).
+-else.
+-ifdef(ERLANG_OTP_VERSION_19).
+-else.
+-define(ERLANG_OTP_VERSION_20_FEATURES, true).
+-endif.
+-endif.
 -endif.
 -endif.
 
--ifdef(TIMESTAMP_ERLANG_NOW).
--type timestamp_type_internal() :: 'erlang_now' | 'os'.
--else.
+-ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 -type timestamp_type_internal() :: 'erlang_timestamp' | 'os' | 'warp'.
+-else.
+-type timestamp_type_internal() :: 'erlang_now' | 'os'.
 -endif.
 
 -record(uuid_state,
@@ -117,7 +108,6 @@
         timestamp_last :: integer() % microseconds
     }).
 
-
 -type cloudi_x_uuid() :: <<_:128>>.
 -type timestamp_type() :: 'erlang' | 'os' | 'warp'.
 -type state() :: #uuid_state{}.
@@ -184,15 +174,7 @@ new(Pid, Options)
 
     % make the version 1 UUID specific to the Erlang node and pid
 
-    % 48 bits for the first MAC address found is included with the
-    % distributed Erlang node name
-    <<NodeD01, NodeD02, NodeD03, NodeD04, NodeD05,
-      NodeD06, NodeD07, NodeD08, NodeD09, NodeD10,
-      NodeD11, NodeD12, NodeD13, NodeD14, NodeD15,
-      NodeD16, NodeD17, NodeD18, NodeD19, NodeD20>> =
-      crypto:hash(sha, erlang:list_to_binary(MacAddress ++
-                                             erlang:atom_to_list(node()))),
-    % later, when the pid format changes, handle the different format
+    % (when the pid format changes, handle the different format)
     ExternalTermFormatVersion = 131,
     PidExtType = 103,
     <<ExternalTermFormatVersion:8,
@@ -201,33 +183,17 @@ new(Pid, Options)
     % 72 bits for the Erlang pid
     <<PidID1:8, PidID2:8, PidID3:8, PidID4:8, % ID (Node specific, 15 bits)
       PidSR1:8, PidSR2:8, PidSR3:8, PidSR4:8, % Serial (extra uniqueness)
-      PidCR1:8                       % Node Creation Count
+      PidCR1:8                                % Node Creation Count
       >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
-    % reduce the 160 bit NodeData checksum to 16 bits
-    NodeByte1 = ((((((((NodeD01 bxor NodeD02)
-                       bxor NodeD03)
-                      bxor NodeD04)
-                     bxor NodeD05)
-                    bxor NodeD06)
-                   bxor NodeD07)
-                  bxor NodeD08)
-                 bxor NodeD09)
-                bxor NodeD10,
-    NodeByte2 = (((((((((NodeD11 bxor NodeD12)
-                        bxor NodeD13)
-                       bxor NodeD14)
-                      bxor NodeD15)
-                     bxor NodeD16)
-                    bxor NodeD17)
-                   bxor NodeD18)
-                  bxor NodeD19)
-                 bxor NodeD20)
-                bxor PidCR1,
+    % 48 bits for the first MAC address found is included with the
+    % distributed Erlang node name to create a node specific value in 16 bits
+    Node32 = cloudi_x_quickrand_hash:jenkins_32([MacAddress,
+                                        erlang:atom_to_binary(node(), utf8)],
+                                       PidCR1),
+    Node16 = (Node32 bsr 16) bxor (Node32 band 16#FFFF),
     % reduce the Erlang pid to 32 bits
-    PidByte1 = PidID1 bxor PidSR4,
-    PidByte2 = PidID2 bxor PidSR3,
-    PidByte3 = PidID3 bxor PidSR2,
-    PidByte4 = PidID4 bxor PidSR1,
+    Pid32 = cloudi_x_quickrand_hash:jenkins_32([PidID1, PidID2, PidID3, PidID4,
+                                       PidSR1, PidSR2, PidSR3, PidSR4]),
     ClockSeq = pseudo_random(16384) - 1,
     TimestampTypeInternal = if
         TimestampType =:= os ->
@@ -245,9 +211,8 @@ new(Pid, Options)
             end
     end,
     TimestampLast = timestamp(TimestampTypeInternal),
-    #uuid_state{node_id = <<NodeByte1:8, NodeByte2:8,
-                            PidByte1:8, PidByte2:8,
-                            PidByte3:8, PidByte4:8>>,
+    #uuid_state{node_id = <<Node16:16/big-unsigned-integer,
+                            Pid32:32/big-unsigned-integer>>,
                 clock_seq = ClockSeq,
                 timestamp_type = TimestampTypeInternal,
                 timestamp_last = TimestampLast}.
@@ -302,16 +267,15 @@ get_v1_time() ->
 -spec get_v1_time(timestamp_type() | state() | cloudi_x_uuid()) ->
     non_neg_integer().
 
--ifdef(TIMESTAMP_ERLANG_NOW).
-% Erlang < 18.0
+-ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 get_v1_time(erlang) ->
-    timestamp(erlang_now);
+    timestamp(erlang_timestamp);
 
 get_v1_time(os) ->
     timestamp(os);
 
 get_v1_time(warp) ->
-    erlang:exit(badarg);
+    timestamp(warp);
 
 get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
     timestamp(TimestampTypeInternal);
@@ -327,15 +291,14 @@ get_v1_time(Value)
     <<Time:60>> = <<TimeHigh:12, TimeMid:16, TimeLow:32>>,
     ((Time - 16#01b21dd213814000) div 10). % microseconds since UNIX epoch
 -else.
-% Erlang >= 18.0
 get_v1_time(erlang) ->
-    timestamp(erlang_timestamp);
+    timestamp(erlang_now);
 
 get_v1_time(os) ->
     timestamp(os);
 
 get_v1_time(warp) ->
-    timestamp(warp);
+    erlang:exit(badarg);
 
 get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
     timestamp(TimestampTypeInternal);
@@ -1265,8 +1228,41 @@ test() ->
            {int_to_hex,1},
            {hex_to_int,1}]}).
 
--ifdef(TIMESTAMP_ERLANG_NOW).
+-ifdef(ERLANG_OTP_VERSION_18_FEATURES).
+timestamp_type_erlang() ->
+    % Erlang >= 18.0
+    true = erlang:function_exported(erlang, system_time, 0),
+    erlang_timestamp.
 
+-ifdef(ERLANG_OTP_VERSION_20_FEATURES).
+timestamp(erlang_timestamp) ->
+    erlang:system_time(microsecond);
+timestamp(os) ->
+    os:system_time(microsecond);
+timestamp(warp) ->
+    erlang:system_time(microsecond).
+-else.
+timestamp(erlang_timestamp) ->
+    erlang:system_time(micro_seconds);
+timestamp(os) ->
+    os:system_time(micro_seconds);
+timestamp(warp) ->
+    erlang:system_time(micro_seconds).
+-endif.
+
+timestamp(erlang_timestamp, TimestampLast) ->
+    TimestampNext = timestamp(erlang_timestamp),
+    if
+        TimestampNext > TimestampLast ->
+            TimestampNext;
+        true ->
+            TimestampLast + 1
+    end;
+timestamp(os, _) ->
+    timestamp(os);
+timestamp(warp, _) ->
+    timestamp(warp).
+-else.
 timestamp_type_erlang() ->
     % Erlang < 18.0
     false = erlang:function_exported(erlang, system_time, 0),
@@ -1283,35 +1279,6 @@ timestamp(erlang_now, _) ->
     timestamp(erlang_now);
 timestamp(os, _) ->
     timestamp(os).
-
--else.
-
-timestamp_type_erlang() ->
-    % Erlang >= 18.0
-    true = erlang:function_exported(erlang, system_time, 0),
-    erlang_timestamp.
-
-timestamp(erlang_timestamp) ->
-    erlang:system_time(micro_seconds);
-timestamp(os) ->
-    {MegaSeconds, Seconds, MicroSeconds} = os:timestamp(),
-    (MegaSeconds * 1000000 + Seconds) * 1000000 + MicroSeconds;
-timestamp(warp) ->
-    erlang:system_time(micro_seconds).
-
-timestamp(erlang_timestamp, TimestampLast) ->
-    TimestampNext = timestamp(erlang_timestamp),
-    if
-        TimestampNext > TimestampLast ->
-            TimestampNext;
-        true ->
-            TimestampLast + 1
-    end;
-timestamp(os, _) ->
-    timestamp(os);
-timestamp(warp, _) ->
-    timestamp(warp).
-
 -endif.
 
 int_to_dec_list(I, N) when is_integer(I), I >= 0 ->
@@ -1373,7 +1340,7 @@ mac_address([{_, L} | Rest]) ->
 
 -ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 pseudo_random(N) ->
-    % assuming exsplus for 58 bits, period 8.31e34
+    % assuming exsp/exsplus for 58 bits, period 8.31e34
     rand:uniform(N).
 -else.
 pseudo_random(N) ->
